SUBROUTINE Q6_total(Q6avg)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  DOUBLE PRECISION :: Q6avg
 

  ! Parameters
  INTEGER,PARAMETER :: n_Q6_neigh = 4
  DOUBLE PRECISION,PARAMETER :: Q6_cutoff = 5.0d0
 
  ! Local
  INTEGER :: i,j,k
  INTEGER :: imol,jmol
  INTEGER :: icell,jcell,jcellndx,jmolndx
  DOUBLE PRECISION :: rxij,ryij,rzij,rij
  INTEGER,DIMENSION(n_mol_tot) :: n_nneigh
  INTEGER,DIMENSION(200,n_mol_tot) :: nneigh_lst
  DOUBLE PRECISION,DIMENSION(4,200,n_mol_tot) :: rij_lst
  INTEGER :: nneigh_temp
  DOUBLE PRECISION,DIMENSION(4) :: rij_temp

  DOUBLE PRECISION,PARAMETER :: pren6 = 1.0d0/64.0d0*dSQRT(3003.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pren5 = 3.0d0/32.0d0*dSQRT(1001.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pren4 = 3.0d0/32.0d0*dSQRT(91.0d0/2.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pren3 = 1.0d0/32.0d0*dSQRT(1365.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pren2 = 1.0d0/64.0d0*dSQRT(1365.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pren1 = 1.0d0/16.0d0*dSQRT(273.0d0/2.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pre0  = 1.0d0/32.0d0*dSQRT(13.0d0/Pi)
  DOUBLE PRECISION,PARAMETER :: pre1  = -pren1
  DOUBLE PRECISION,PARAMETER :: pre2  =  pren2
  DOUBLE PRECISION,PARAMETER :: pre3  = -pren3
  DOUBLE PRECISION,PARAMETER :: pre4  =  pren4
  DOUBLE PRECISION,PARAMETER :: pre5  = -pren5
  DOUBLE PRECISION,PARAMETER :: pre6  =  pren6
  DOUBLE PRECISION :: pre(-6:6), post(-6:6)
  DOUBLE PRECISION :: cost(6),sint(6)
  DOUBLE PRECISION :: cosp(6), sinp(6)
  COMPLEX*16 :: Y(-6:6)
  COMPLEX*16 :: sum_Y(-6:6),sum_Q6(-6:6)
 

  ! Initialize arrays
  n_nneigh = 0
  nneigh_lst = 0
  rij  = 0.0d0
  
  ! Construct short range neighbor list
  DO imol = 1,n_mol_tot-1
    DO jmol = imol + 1,n_mol_tot
 
      ! Calculate the COM separation vector
      rxij = rx(jmol) - rx(imol)
      ryij = ry(jmol) - ry(imol)
      rzij = rz(jmol) - rz(imol)

      ! Apply the periodic boundary conditions
      rxij = rxij - dNINT(rxij/box(1))*box(1)
      ryij = ryij - dNINT(ryij/box(2))*box(2)
      rzij = rzij - dNINT(rzij/box(3))*box(3)

      ! Calculate the square of the COM scalar separation distance
      rij = dSQRT(rxij**2 + ryij**2 + rzij**2)

      ! If they are within the cutoff distance   
      IF(rij .LT. Q6_cutoff) THEN

        ! Update information for imol

        ! Update the number of neighbors for imol
        n_nneigh(imol) = n_nneigh(imol) + 1
   
        ! Include jmol as a neighbor of imol
        nneigh_lst(n_nneigh(imol),imol)  = jmol

        ! Store the distance components for imol
        rij_lst(1, n_nneigh(imol), imol) = rxij
        rij_lst(2, n_nneigh(imol), imol) = ryij
        rij_lst(3, n_nneigh(imol), imol) = rzij
        rij_lst(4, n_nneigh(imol), imol) = rij

        ! Update information for jmol
  
        ! Update the number of neighbors for jmol
        n_nneigh(jmol) = n_nneigh(jmol) + 1

        ! Include imol as a neighbor of jmol
        nneigh_lst(n_nneigh(jmol),jmol) = imol

        ! Store the distance components for jmol
        rij_lst(:,n_nneigh(jmol),jmol) = rij_lst(:,n_nneigh(imol),imol)
        
      ENDIF
        
    ENDDO
  ENDDO

  ! Check the minimum number of neighbors are present
  IF(MINVAL(n_nneigh(:)) .LT. n_Q6_neigh) THEN
    WRITE(*,'(A,I5,A,I3,A)') 'FATAL ERROR: ', imol, ' HAS LESS THAN ', n_Q6_neigh,' NEAREST NEIGHBORS IN SUBROUTINE Q6, ADJUST Q6_CUTOFF'
    STOP
  ENDIF

  ! Check that the maximum number of neighbors is not exceeded
  IF(MAXVAL(n_nneigh(:)) .GT. 200) THEN
    WRITE(*,'(A,I5,A)') 'FATAL ERROR: ', imol, ' HAS MORE THAN 200 NEAREST NEIGHBORS IN SUBROUTINE Q6'
    STOP
  ENDIF

  ! Sort the neighbors using insertion sort if using only the closest n_Q6_neigh neighbors
  IF(n_Q6_neigh .NE. 0 ) THEN
    DO imol = 1,n_mol_tot
      DO k = 2,n_nneigh(imol)
        nneigh_temp = nneigh_lst(k,imol)
        rij_temp(:) = rij_lst(:,k,imol)
        DO j = k-1,1,-1
          IF(rij_lst(4,j,imol) .LE. rij_temp(4)) GOTO 10
          nneigh_lst(j + 1, imol) = nneigh_lst(j,imol)
          rij_lst(:,j + 1,imol) = rij_lst(:, j, imol)
        ENDDO
        jmol = 0
        10 CONTINUE
        nneigh_lst(j + 1, imol) = nneigh_temp
        rij_lst(:,j + 1, imol) = rij_temp(:)
      ENDDO
      n_nneigh(imol) = n_Q6_neigh
    ENDDO
  ENDIF


  ! Calculate Q6
  
  ! Initialize the sum
  sum_Q6 = 0.0d0

  ! Loop over the molecules
  DO imol = 1,n_mol_tot

    ! Initialize the harmonic sum
    sum_Y = 0.0d0
  
    ! Loop over the 4 nearest neighbors of imol
    DO j = 1,n_nneigh(imol)

      ! Calculate cos(theta) and sin(theta)
      cost(1) = rij_lst(3,j,imol)/rij_lst(4,j,imol)
      sint(1) = (1.0d0-cost(1)*cost(1))**0.5d0

      ! Calculate cos(phi) and sin(phi)
      IF (dABS(sint(1)) .LT. 1d-9) THEN
        cosp(1) = 1.0d0
        sinp(1) = 0.0d0
      ELSE
        cosp(1) = rij_lst(1,j,imol)/(rij_lst(4,j,imol)*sint(1))
        sinp(1) = rij_lst(2,j,imol)/(rij_lst(4,j,imol)*sint(1))
      ENDIF

      ! Calculate powers of sin(theta)
      sint(2) = sint(1)*sint(1)
      sint(3) = sint(2)*sint(1)
      sint(4) = sint(2)*sint(2)
      sint(5) = sint(3)*sint(2)
      sint(6) = sint(3)*sint(3)

      ! Calculate powers of cos(theta)
      cost(2) = cost(1)*cost(1)
      cost(3) = cost(2)*cost(1)
      cost(4) = cost(2)*cost(2)
      cost(5) = cost(3)*cost(2)
      cost(6) = cost(3)*cost(3)

      ! Calculate sin/cos 2x
      sinp(2) = 2.0d0*sinp(1)*cosp(1)
      cosp(2) = 2.0d0*cosp(1)**2 - 1.0d0

      ! Calculate sin/cos 3x
      sinp(3) = sinp(2)*cosp(1) + cosp(2)*sinp(1)
      cosp(3) = cosp(2)*cosp(1) - sinp(2)*sinp(1)

      ! Calculate sin/cos 4x
      sinp(4) = 2.0d0*sinp(2)*cosp(2)
      cosp(4) = 2.0d0*cosp(2)**2 - 1.0d0

      ! Calculate sin/cos 5x
      sinp(5) = sinp(3)*cosp(2) + cosp(3)*sinp(2)
      cosp(5) = cosp(3)*cosp(2) - sinp(3)*sinp(2)

      ! Calculate sin/cos 6x
      sinp(6) = 2.0d0*sinp(3)*cosp(3)
      cosp(6) = 2.0d0*cosp(3)**2 - 1.0d0

      ! Calculate post factor
      post(-6) = sint(6)
      post(-5) = sint(5)*cost(1)
      post(-4) = sint(4)*(11.0d0*cost(2)-1.0d0)
      post(-3) = sint(3)*(11.0d0*cost(3)-3.0d0*cost(1))
      post(-2) = sint(2)*(33.0d0*cost(4)-18.0d0*cost(2)+1.0d0)
      post(-1) = sint(1)*(33.0d0*cost(5)-30.0d0*cost(3)+5.0d0*cost(1))
      post(0) = (231.0d0*cost(6)-315.0d0*cost(4)+105.0d0*cost(2)-5.0d0)
      post(1) = post(-1)
      post(2) = post(-2)
      post(3) = post(-3)
      post(4) = post(-4)
      post(5) = post(-5)
      post(6) = post(-6)

      ! Calculate the harmonics
      Y(-6)=pren6*dCMPLX(cosp(6),-sinp(6))*post(-6)
      Y(-5)=pren5*dCMPLX(cosp(5),-sinp(5))*post(-5)
      Y(-4)=pren4*dCMPLX(cosp(4),-sinp(4))*post(-4)
      Y(-3)=pren3*dCMPLX(cosp(3),-sinp(3))*post(-3)
      Y(-2)=pren2*dCMPLX(cosp(2),-sinp(2))*post(-2)
      Y(-1)=pren1*dCMPLX(cosp(1),-sinp(1))*post(-1)
      Y(0)=pre0*post(0)
      Y(1)=pre1*dCMPLX(cosp(1),sinp(1))*post(1)
      Y(2)=pre2*dCMPLX(cosp(2),sinp(2))*post(2)
      Y(3)=pre3*dCMPLX(cosp(3),sinp(3))*post(3)
      Y(4)=pre4*dCMPLX(cosp(4),sinp(4))*post(4)
      Y(5)=pre5*dCMPLX(cosp(5),sinp(5))*post(5)
      Y(6)=pre6*dCMPLX(cosp(6),sinp(6))*post(6)

      ! Update the harmonic (Y) sum
      sum_Y = sum_Y + Y

    ENDDO

    ! Update the Q6 sum
    sum_Q6 = sum_Q6 + sum_Y/DBLE(n_nneigh(imol))

  ENDDO
  
  ! Normalize the Q6 sum
  sum_Q6 = sum_Q6/DBLE(n_mol_tot)

  ! Calculate the average Q6
  Q6avg = 0.0d0
  DO i = -6,6
    Q6avg = Q6avg + sum_Q6(i)*dCONJG(sum_Q6(i))
  ENDDO
  Q6avg = dSQRT(Q6avg)
  
  RETURN

END SUBROUTINE Q6_total
