SUBROUTINE dump(selector)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'
  DOUBLE PRECISION :: accm, accv
 
  ! Passed
  INTEGER :: selector

  ! Local
  INTEGER :: itype, jtype, i

  ! Select statistics operation
  SELECT CASE (selector)


    ! Open files for writing
    CASE(0)

      ! If running weighted ensemble simulation, open cv dump file
      IF(wensmbl .EQ. 1) CALL dump_cvs(0)

     ! If trajectory is being dumped, open files
      IF(dump_traj .GT. 0) CALL dump_trajectory(0)


    ! Write to file/screen
    CASE(2)

      ! Check if CVs should be dumped file
      IF((wensmbl .EQ. 1) .AND. (MOD(istep,n_cvdump) .EQ. 0)) CALL dump_cvs(2)

      ! Check if trajectory should be dumped
      IF((dump_traj .GT. 0) .AND. (MOD((iblock-1)*n_steps + istep,n_traj) .EQ. 0)) CALL dump_trajectory(2)

      ! Print out at the end of every block
      IF(istep .EQ. n_steps) THEN
          IF(my_rank .EQ. 1) THEN
            ! Loop over the molecule types
            DO itype = 1,n_types
              accm = 0.0d0
              accv = 0.0d0
              IF(multimove_stat(1) .GT. 0) accm = DBLE(multimove_stat(2))/DBLE(multimove_stat(1))
              IF(vol_stat(1) .GT. 0) accv =  DBLE(vol_stat(2))/DBLE(vol_stat(1))
              WRITE(*,'(A,I0,4F16.6)')'Block ',iblock, R*energy/DBLE(n_mol_tot), (DBLE(n_mol_tot)/vol)*10.0d0*mol_mass(1)/6.0221413d0, accv,accm
            ENDDO
          ENDIF
      ENDIF


    ! Close files for writing
    CASE(4)

      ! Check if CVs file should be closed
      IF(wensmbl .EQ. 1) CALL dump_cvs(4)

      ! Check if trajectory files should be closed
      IF(dump_traj .GT. 1) CALL dump_trajectory(4)


    ! Throw an error
    CASE DEFAULT

    WRITE(*,*) 'FATAL ERROR: INVALID FLAG TYPE IN DUMP SUBROUTINE'
      STOP


   END SELECT


   RETURN


END SUBROUTINE dump
