SUBROUTINE engforce_ij(imol,jmol,engij,fis,fjs)

  USE global
  IMPLICIT NONE

  ! Passed
  INTEGER :: imol, jmol
  DOUBLE PRECISION,DIMENSION(2) :: engij
  DOUBLE PRECISION,DIMENSION(3,n_sites_max) :: fis,fjs

  ! Local 
  INTEGER :: itype,jtype
  INTEGER :: isite,jsite
  DOUBLE PRECISION :: lj_factor,f_factor
  DOUBLE PRECISION :: engij_coul, fxij_coul, fyij_coul, fzij_coul
  DOUBLE PRECISION :: rijc, rijsqc, rxijc, ryijc, rzijc
  DOUBLE PRECISION :: rxis, ryis, rzis
  DOUBLE PRECISION :: rijs, rijsqs, rxijs, ryijs, rzijs
  DOUBLE PRECISION :: fxij,fyij,fzij
  DOUBLE PRECISION :: pchar
  DOUBLE PRECISION,PARAMETER :: RL=2.0160d0, RU=3.12870d0, DR3 = (RU-RL)**3
  DOUBLE PRECISION :: SR,dSR
  DOUBLE PRECISION :: two_invsqrtpi = 2.0d0/dSQRT(Pi)

  ! Initialize the energy contribution
  engij = 0.0d0
  fis = 0.0d0
  fjs = 0.0d0
  engij_coul = 0.0d0  
  pchar = 0.0d0

  ! Get the molecule types
  itype = mol_type(imol)
  jtype = mol_type(jmol)
 

  ! Oxygen - Oxygen interactions

  ! Calculate the Oxygen-Oxygen separation vector 
  rxijc = rx_s(1,jmol) - rx_s(1,imol)
  ryijc = ry_s(1,jmol) - ry_s(1,imol)
  rzijc = rz_s(1,jmol) - rz_s(1,imol)

  ! Apply the periodic boundary conditions
  rxijc = rxijc - dNINT(rxijc/box(1))*box(1)
  ryijc = ryijc - dNINT(ryijc/box(2))*box(2)
  rzijc = rzijc - dNINT(rzijc/box(3))*box(3)

  ! Calculate the square of the COM scalar separation distance
  rijsqc = rxijc**2 + ryijc**2 + rzijc**2


  ! Check the COM cutoff distance
  IF(rijsqc .GE. rmaxsq(jtype,itype)) RETURN


  ! Calculate the distance
  rijc = dSQRT(rijsqc)


  ! Compute switching function 
  IF(rijc .GT. RU) THEN
    SR  = 1.0d0
    dSR = 0.0d0
  ELSEIF(rijc .LE. RL) THEN
     SR = 0.0d0
    dSR = 0.0d0
  ELSE
    SR  = ((rijc - RL)**2)*(3.0d0*RU - RL - 2.0d0*rijc)/DR3
    dSR = -6.0d0*(rijc - RL)*(rijc - RU)/DR3
  ENDIF
    

  ! Charge - Charge interactions

  ! Loop over imol's sites
  DO isite = 2,n_sites(itype)

    ! Get the isite's position vectors
    rxis = rx_s(isite,imol)
    ryis = ry_s(isite,imol)
    rzis = rz_s(isite,imol)

    ! Loop over jmol's sites
    DO jsite = 2,n_sites(jtype)

      ! Calculate the site separation vector 
      rxijs = rx_s(jsite,jmol) - rxis
      ryijs = ry_s(jsite,jmol) - ryis
      rzijs = rz_s(jsite,jmol) - rzis

      ! Apply the periodic boundary conditions
      rxijs = rxijs - dNINT(rxijs/box(1))*box(1)
      ryijs = ryijs - dNINT(ryijs/box(2))*box(2)
      rzijs = rzijs - dNINT(rzijs/box(3))*box(3)

      ! Calculate the square of the site scalar separation distance
      rijsqs = rxijs**2 + ryijs**2 + rzijs**2

      ! Calculate the site scalar separation distance
      rijs = dSQRT(rijsqs)

      ! Compute the electrostatic energy

      ! Compute the screened electrostatic energy
      engij(2) = engij(2) + Pe*qsq(jsite,isite,jtype,itype)*dERFC(kappa*rijs)/rijs

      ! Compute the direct electrostatic energy
      engij_coul = engij_coul + Pe*qsq(jsite,isite,jtype,itype)/rijs


      ! Compute the electrostatic forces

      ! Compute the screened electrostatic forces
      f_factor =  Pe*qsq(jsite,isite,jtype,itype)*(dERFC(kappa*rijs)/rijs + two_invsqrtpi*kappa*dEXP(-kappa**2*rijsqs))/rijsqs

      !  Apply short-range correction to f_factor, based on SQ
      f_factor = f_factor - Pe*(1.0d0 - SR)*(qsq(jsite,isite,jtype,itype)/rijs)/rijsqs
 
      ! Compute the x,y,z forces
      fxij = f_factor*rxijs
      fyij = f_factor*ryijs
      fzij = f_factor*rzijs

      ! Update LJ forces on imol
      fis(1,isite) =  fis(1,isite) - fxij
      fis(2,isite) =  fis(2,isite) - fyij
      fis(3,isite) =  fis(3,isite) - fzij

      ! Update LJ forces on jmol
      fjs(1,jsite) =  fjs(1,jsite) + fxij
      fjs(2,jsite) =  fjs(2,jsite) + fyij
      fjs(3,jsite) =  fjs(3,jsite) + fzij

  
    ! End loop over jmol's sites
    ENDDO

  ! End loop over imol's sites
  ENDDO

  ! Correct the electrostatic energy using SR
  engij(2) = engij(2) - (1.0d0 - SR)*engij_coul

  ! Compute the Lenanrd-Jones Energy

  ! Compute the Lennard-Jones energy factor
  lj_factor = (sigsq(1,1,jtype,itype)/rijsqc)**3

  ! Compute the Lennard-Jones energy
  engij(1) = engij(1) + 4.0d0*eps(1,1,jtype,itype)*lj_factor*(lj_factor-1.0d0)

  ! Compute the Lennard-Jones Forces

  ! Compute the Lennard-Jones force factor [K/A^2]
  f_factor = 48.0d0*eps(1,1,jtype,itype)*lj_factor*(lj_factor-0.5d0)/rijsqc 

  ! Apply short-range correction to f_factor, based on SQ
  f_factor = f_factor -  engij_coul*dSR/rijc

  ! Compute the x,y,z forces [K/A]
  fxij = f_factor*rxijc
  fyij = f_factor*ryijc
  fzij = f_factor*rzijc

  ! Update LJ forces on imol
  fis(1,1) =  fis(1,1) - fxij
  fis(2,1) =  fis(2,1) - fyij
  fis(3,1) =  fis(3,1) - fzij

  ! Update LJ forces on jmol
  fjs(1,1) =  fjs(1,1) + fxij
  fjs(2,1) =  fjs(2,1) + fyij
  fjs(3,1) =  fjs(3,1) + fzij


  RETURN

END SUBROUTINE engforce_ij
