SUBROUTINE engforce_total(eng_tot,frc_tot,trq_tot)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'
 
  ! Passed
  DOUBLE PRECISION :: eng_tot

  ! Local 
  INTEGER :: imol,itype,jmol,jtype,n_total
  DOUBLE PRECISION :: engewld
  DOUBLE PRECISION,DIMENSION(2) :: engij
  DOUBLE PRECISION,DIMENSION(3,n_sites_max) :: fis,fjs,drsc
  DOUBLE PRECISION,DIMENSION(3,n_mol_max) :: frc_tot, trq_tot
  DOUBLE PRECISION,DIMENSION(3,n_sites_max,n_mol_max) :: fis_sum, fis_sum_ewld
  DOUBLE PRECISION,DIMENSION(3):: frc_upd, trq_upd
  DOUBLE PRECISION :: eng_upd

  ! Initialize the energy  contribution
  eng_tot = 0.0d0
  frc_tot = 0.0d0
  trq_tot = 0.0d0
  fis_sum = 0.0d0
  fis_sum_ewld = 0.0d0

  ! Calculate the contribution from other molecule

  ! Loop over all the pairs of molecules
  DO imol = 1,n_mol_tot-1
    DO jmol = imol + 1,n_mol_tot

      ! Calculate the dispersion energy
      IF(ewld .EQ. 1) THEN
       CALL engforce_ij(imol,jmol,engij,fis,fjs)
      ELSE
        CALL engforce_ij_rf(imol,jmol,engij,fis,fjs)
      ENDIF 

      ! Update the total energy

      ! Dispersion contribution
      eng_tot = eng_tot + engij(1)

      ! Electrostatic contribution
      eng_tot = eng_tot + engij(2)

      ! Update the force on imol and jmol
      fis_sum(:,:,imol) = fis_sum(:,:,imol) + fis(:,:)
      fis_sum(:,:,jmol) = fis_sum(:,:,jmol) + fjs(:,:)

    ENDDO
  ENDDO

  ! Check for Ewald summation
  IF(ewld .EQ. 1) THEN

    ! Calculate the total Ewald contribution
    CALL ewld_total(engewld, fis_sum_ewld)

    ! Update the total energy
    eng_tot = eng_tot + engewld
  
    ! Update the forces
    fis_sum(:,:,:) = fis_sum(:,:,:) + fis_sum_ewld(:,:,:)

  ENDIF

  ! Calculate the net force and torque on each molecule
  DO imol = 1,n_mol_tot
  
    ! Convert units of the force from K/A to [(g A)/(mol ps^2)]
    fis_sum(1,:,imol) = Fc*fis_sum(1,:,imol)
    fis_sum(2,:,imol) = Fc*fis_sum(2,:,imol)
    fis_sum(3,:,imol) = Fc*fis_sum(3,:,imol)

    !Calculate net force on imol [(g A)/(mol ps^2)] 
    frc_tot(1,imol) = SUM(fis_sum(1,:,imol))
    frc_tot(2,imol) = SUM(fis_sum(2,:,imol))
    frc_tot(3,imol) = SUM(fis_sum(3,:,imol))

    ! Compute the distance of each site from the center of mass A
    drsc(1,:) =  (rx_s(:,imol) - rx(imol))
    drsc(2,:) =  (ry_s(:,imol) - ry(imol))
    drsc(3,:) =  (rz_s(:,imol) - rz(imol))

    ! Calculate torques in space frame [(g A^2)/(mol ps^2)]
    trq_tot(1,imol) = SUM(drsc(2,:)*fis_sum(3,:,imol) - drsc(3,:)*fis_sum(2,:,imol))
    trq_tot(2,imol) = SUM(drsc(3,:)*fis_sum(1,:,imol) - drsc(1,:)*fis_sum(3,:,imol))
    trq_tot(3,imol) = SUM(drsc(1,:)*fis_sum(2,:,imol) - drsc(2,:)*fis_sum(1,:,imol))
 
  ENDDO

  RETURN

END SUBROUTINE engforce_total
