SUBROUTINE ewld_total(ewldtot, fis_sum_ewld)

  USE global
  IMPLICIT NONE 
  INCLUDE 'mpif.h'

 
  ! Passed
  DOUBLE PRECISION :: ewldtot
  DOUBLE PRECISION,DIMENSION(3,n_sites_max,n_mol_max) :: fis_sum_ewld
 
  ! Local
  INTEGER :: ksq,kx,ky,kz,k
  INTEGER :: imol,itype,ichrg,isite,jchrg,jsite
  INTEGER :: iatom
  DOUBLE PRECISION :: rx_charge,ry_charge,rz_charge
  DOUBLE PRECISION,DIMENSION(3) :: two_Pibox
  DOUBLE PRECISION :: ewldself, ewldsurf
  DOUBLE PRECISION :: rkx,rky,rkz,kr,f_factor
  DOUBLE PRECISION :: rxijs,ryijs,rzijs,rijs
  DOUBLE PRECISION,DIMENSION(3,n_sites_max) :: f_ewld_self 

  ! Calculate the total reciprocal energy  

  ! Initialize variables
  ewldtot = 0.0d0
  fis_sum_ewld = 0.0d0
  ewldself = 0.0d0
  ewldsurf = 0.0d0
  ewld_surfc(:) = 0.0d0
  two_Pibox(:) = 2.0d0*Pi/box(:)

  !  Fluid contributions

  ! Loop over all of the molecules in ibox
  DO imol = 1,n_mol_tot
  
    ! Get imol's type
    itype = mol_type(imol)

    ! Loop over the charges on the molecule
    DO ichrg = 1,n_ions(itype)

      ! Get the site number from the charge list
      isite = ion_sites(ichrg,itype)
     
      ! Get the site positions
      rx_charge = rx_s(isite,imol)
      ry_charge = ry_s(isite,imol)
      rz_charge = rz_s(isite,imol)

      ! Calculate the reciprocal vector components for k = 0
      eikx(ichrg,imol,0) = (1.0d0, 0.0d0)
      eiky(ichrg,imol,0) = (1.0d0, 0.0d0) 
      eikz(ichrg,imol,0) = (1.0d0, 0.0d0)

      ! Calculate the reciprocal vector components for k = 1
      eikx(ichrg,imol,1) = dCMPLX(dCOS(two_Pibox(1)*rx_charge),dSIN(two_Pibox(1)*rx_charge))
      eiky(ichrg,imol,1) = dCMPLX(dCOS(two_Pibox(2)*ry_charge),dSIN(two_Pibox(2)*ry_charge))
      eikz(ichrg,imol,1) = dCMPLX(dCOS(two_Pibox(3)*rz_charge),dSIN(two_Pibox(3)*rz_charge))

      ! Calculate the reciprocal vector components for k = -1 (NOTE : kx > 0)
      eiky(ichrg,imol,-1) = dCONJG(eiky(ichrg,imol,1))
      eikz(ichrg,imol,-1) = dCONJG(eikz(ichrg,imol,1))

      ! Calculate the surface potential components
      ewld_surfc(1) = ewld_surfc(1) + q(isite,isite,itype,itype)*rx_charge
      ewld_surfc(2) = ewld_surfc(2) + q(isite,isite,itype,itype)*ry_charge 
      ewld_surfc(3) = ewld_surfc(3) + q(isite,isite,itype,itype)*rz_charge


    ENDDO
  ENDDO
 
  ! Calculate the remaining reciprocal vector components by recursion (x-component)
  DO kx = 2,k_max(1)
    DO imol = 1,n_mol_tot
      itype = mol_type(imol)
      DO ichrg = 1,n_ions(itype)

        ! x-component
        eikx(ichrg,imol,kx) = eikx(ichrg,imol,kx-1)*eikx(ichrg,imol,1) 
         
      ENDDO
    ENDDO
  ENDDO

  ! Calculate the remaining reciprocal vector components by recursion (y-component)
  DO ky = 2,k_max(2)
    DO imol = 1,n_mol_tot
      itype = mol_type(imol)
      DO ichrg = 1,n_ions(itype)

        ! y-component
        eiky(ichrg,imol,ky) = eiky(ichrg,imol,ky-1)*eiky(ichrg,imol,1)
        eiky(ichrg,imol,-ky) = dCONJG(eiky(ichrg,imol,ky))

      ENDDO
    ENDDO
  ENDDO

  ! Calculate the remaining reciprocal vector components by recursion (z-component)
  DO kz = 2,k_max(3)
    DO imol = 1,n_mol_tot
      itype = mol_type(imol)
      DO ichrg = 1,n_ions(itype)

        ! z-component
        eikz(ichrg,imol,kz) = eikz(ichrg,imol,kz-1)*eikz(ichrg,imol,1)
        eikz(ichrg,imol,-kz) = dCONJG(eikz(ichrg,imol,kz))

      ENDDO
    ENDDO
  ENDDO



  ! Calculate the structure factor and reciprocal space energy

  ! Zero the structure factor
  skewld(:) = (0.0d0, 0.0d0)

  ! Fluid contribution

  ! Loop over kvectors
  DO k = 1,n_k
  
    ! Get kvector
    kx = kvx(k)
    ky = kvy(k)
    kz = kvz(k)

    ! Fluid contribution

    ! Loop over the molecules
    DO imol = 1,n_mol_tot

      ! Get imol's type
      itype = mol_type(imol)

      ! Loop over imol's charges
      DO ichrg = 1,n_ions(itype)

        ! Get the site number of the charge
        isite = ion_sites(ichrg,itype)

        ! Calculate the structure factor
         skewld(k) = skewld(k) + q(isite,isite,itype,itype)*eikx(ichrg,imol,kx)*eiky(ichrg,imol,ky)*eikz(ichrg,imol,kz)

       ENDDO
     ENDDO
   ENDDO


   ! Calculate the reciprocal space energy
   
   ! Loop over kvectors
   DO k = 1,n_k
     ewldtot = ewldtot + k_vec(k) * dCONJG(skewld(k))*skewld(k)
   ENDDO


  ! Calculate the spurious self energy contribution (fluid)
  DO itype = 1,n_types
    ewldself = ewldself + n_mol(itype)*ewld_self(itype)
  ENDDO

  ! Calculate the surface contribution
  IF(ewld_vac .EQ. 1) ewldsurf = (2.0d0*Pi)/(3.0d0*vol)*(ewld_surfc(1)**2  + ewld_surfc(2)**2 + ewld_surfc(3)**2)
 
  ! Calculate the final reciprocal space energy and convert to units of Kelvin
  ewldtot = Pe*(ewldtot - ewldself + ewldsurf)

  ! Compute ewald forces
  DO k = 1,n_k

    ! Get kvector
    kx = kvx(k)
    ky = kvy(k)
    kz = kvz(k)

    ! Calculate rk components
    rkx = DBLE(kx)*two_Pibox(1)
    rky = DBLE(ky)*two_Pibox(2)
    rkz = DBLE(kz)*two_Pibox(3)

    ! Loop over all of the molecules in ibox
    DO imol = 1,n_mol_tot

      ! Get imol's type
      itype = mol_type(imol)

      ! Loop over the charges on the molecule
      DO ichrg = 1,n_ions(itype)

        ! Get the site number from the charge list
        isite = ion_sites(ichrg,itype)

        ! Get the site positions
        rx_charge = rx_s(isite,imol)
        ry_charge = ry_s(isite,imol)
        rz_charge = rz_s(isite,imol)

        ! Calculate the dot product
        kr = rkx*rx_charge + rky*ry_charge + rkz*rz_charge

        ! Calculate the force factor
        f_factor = 2.0d0*q(isite,isite,itype,itype)*k_vec(k)*AIMAG(dCONJG(skewld(k))*dCMPLX(dCOS(kr),dSIN(kr)))

        ! Add the force contributions
        fis_sum_ewld(1,isite,imol) = fis_sum_ewld(1,isite,imol) + f_factor*rkx
        fis_sum_ewld(2,isite,imol) = fis_sum_ewld(2,isite,imol) + f_factor*rky
        fis_sum_ewld(3,isite,imol) = fis_sum_ewld(3,isite,imol) + f_factor*rkz

      ENDDO
    ENDDO
  ENDDO


  DO imol = 1,n_mol_tot

    ! Get imol's type
    itype = mol_type(imol)

    ! Zero the force
    f_ewld_self = 0.0d0

    ! Loop over all charge pairs
    DO ichrg = 1,n_ions(itype) - 1
      isite = ion_sites(ichrg,itype)
      DO jchrg = ichrg + 1, n_ions(itype)
        jsite = ion_sites(jchrg,itype)

        ! Calculate the separation vector between the two sites
        rxijs = rx_s(jsite,imol) - rx_s(isite,imol)
        ryijs = ry_s(jsite,imol) - ry_s(isite,imol)
        rzijs = rz_s(jsite,imol) - rz_s(isite,imol)

        ! Calculate scalar separation distance
        rijs = dSQRT(rxijs**2 + ryijs**2 + rzijs**2)

        ! Calculate the force factor
        f_factor = q(isite,isite,itype,itype)*q(jsite,jsite,itype,itype)*(dERF(kappa*rijs)- 2.0d0/dSQRT(Pi)*kappa*rijs*dEXP(-kappa**2*rijs**2))/rijs**3

        ! Add  force to isite
        f_ewld_self(1,isite) =  f_ewld_self(1,isite) + rxijs*f_factor
        f_ewld_self(2,isite) =  f_ewld_self(2,isite) + ryijs*f_factor
        f_ewld_self(3,isite) =  f_ewld_self(3,isite) + rzijs*f_factor

        ! Subtract force from jsite
        f_ewld_self(1,jsite) =  f_ewld_self(1,jsite) - rxijs*f_factor
        f_ewld_self(2,jsite) =  f_ewld_self(2,jsite) - ryijs*f_factor
        f_ewld_self(3,jsite) =  f_ewld_self(3,jsite) - rzijs*f_factor

      ENDDO
    ENDDO

    ! Remove the self contribution
    fis_sum_ewld(1,:,imol) = fis_sum_ewld(1,:,imol) + f_ewld_self(1,:)
    fis_sum_ewld(2,:,imol) = fis_sum_ewld(2,:,imol) + f_ewld_self(2,:)
    fis_sum_ewld(3,:,imol) = fis_sum_ewld(3,:,imol) + f_ewld_self(3,:)

  ENDDO


  ! Correct for boundary condition
  IF(ewld_vac .EQ. 2) THEN

    ! Correct for self interactions
    DO imol = 1,n_mol_tot

      ! Get imol's type
      itype = mol_type(imol)

      ! Loop over the charges on the molecule
      DO ichrg = 1,n_ions(itype)

        ! Get the site number from the charge list
        isite = ion_sites(ichrg,itype)

        ! Subtract the self contribution
        fis_sum_ewld(1,isite,imol) = fis_sum_ewld(1,isite,imol)  - 4.0d0*Pi*q(isite,isite,itype,itype)*ewld_surfc(1)/(3.0d0*vol)
        fis_sum_ewld(2,isite,imol) = fis_sum_ewld(2,isite,imol)  - 4.0d0*Pi*q(isite,isite,itype,itype)*ewld_surfc(2)/(3.0d0*vol)
        fis_sum_ewld(3,isite,imol) = fis_sum_ewld(3,isite,imol)  - 4.0d0*Pi*q(isite,isite,itype,itype)*ewld_surfc(3)/(3.0d0*vol)

      ENDDO

    ENDDO

  ENDIF

  ! Convert the reciprocal space force to K/A
  fis_sum_ewld = Pe*fis_sum_ewld
  
  RETURN

END SUBROUTINE ewld_total
