SUBROUTINE global_allocate(selector)
  !OPT
  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  CHARACTER(LEN=*) :: selector

  ! Local
  INTEGER :: itype, jtype, i
  INTEGER :: ierr


  ! Initialize ierr
  ierr = 0

  ! Select array group to allocate
  SELECT CASE (selector)

    ! Box dimension
    CASE('box') 

      ! Allocate box dimensions
      ALLOCATE(box(3), STAT=ierr)

      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATA ERROR: ALLOCATION OF ARRAY FAILED IN SETUP_ALLOCATE SUBROUTINE'
        STOP
      ENDIF

    ! System variables
    CASE('system')

      ! Allocate system arrays
 
      ! Molecular topology
      ALLOCATE(n_mol(n_types), STAT=ierr)
      ALLOCATE(n_sites(n_types), STAT=ierr)
      ALLOCATE(n_disp(n_types), STAT=ierr)
      ALLOCATE(n_ions(n_types), STAT=ierr)
      ALLOCATE(mol_type(n_mol_max), STAT=ierr)
      ALLOCATE(mol_name(n_types), STAT=ierr)
      ALLOCATE(site_name(n_sites_max,n_types), STAT=ierr)
      ALLOCATE(disp_sites(n_sites_max,n_types),STAT=ierr)
      ALLOCATE(ion_sites(n_sites_max,n_types), STAT=ierr)
      ALLOCATE(site_mass(n_sites_max,n_types),STAT=ierr)
      ALLOCATE(mol_mass(n_types), STAT=ierr)
      ALLOCATE(n_rot_dof(n_types), STAT=ierr)
      ALLOCATE(inrta_tnsr(3,3,n_types), STAT=ierr)
      ALLOCATE(inrta_tnsr_inv(3,3,n_types), STAT=ierr)
      AlLOCATE(rx(n_mol_max), STAT=ierr)
      AlLOCATE(ry(n_mol_max), STAT=ierr)
      AlLOCATE(rz(n_mol_max), STAT=ierr)
      AlLOCATE(vx(n_mol_max), STAT=ierr)
      AlLOCATE(vy(n_mol_max), STAT=ierr)
      AlLOCATE(vz(n_mol_max), STAT=ierr)
      AlLOCATE(wx(n_mol_max), STAT=ierr)
      AlLOCATE(wy(n_mol_max), STAT=ierr)
      AlLOCATE(wz(n_mol_max), STAT=ierr)
      AlLOCATE(q0(n_mol_max), STAT=ierr)
      AlLOCATE(q1(n_mol_max), STAT=ierr)
      AlLOCATE(q2(n_mol_max), STAT=ierr)
      AlLOCATE(q3(n_mol_max), STAT=ierr)
      AlLOCATE(p0(n_mol_max), STAT=ierr)
      AlLOCATE(p1(n_mol_max), STAT=ierr)
      AlLOCATE(p2(n_mol_max), STAT=ierr)
      AlLOCATE(p3(n_mol_max), STAT=ierr)
      ALLOCATE(rx_s(n_sites_max,n_mol_max), STAT=ierr)
      ALLOCATE(ry_s(n_sites_max,n_mol_max), STAT=ierr)
      ALLOCATE(rz_s(n_sites_max,n_mol_max), STAT=ierr)
      ALLOCATE(rx_i(n_sites_max,n_types), STAT=ierr)
      ALLOCATE(ry_i(n_sites_max,n_types), STAT=ierr)
      ALLOCATE(rz_i(n_sites_max,n_types), STAT=ierr)
      ALLOCATE(force(3,n_mol_max), STAT=ierr)
      ALLOCATE(torque(3,n_mol_max), STAT=ierr)

      ! Potential parameters
      ALLOCATE(eps(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(sig(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(alpha(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(q(n_sites_max,n_sites_max,n_types,n_types), STAT =ierr)
      ALLOCATE(sigsq(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(qsq(n_sites_max,n_sites_max,n_types,n_types), STAT =ierr)
      ALLOCATE(rmin(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(rmax(n_types,n_types), STAT=ierr)
      ALLOCATE(rminsq(n_sites_max,n_sites_max,n_types,n_types), STAT=ierr)
      ALLOCATE(rmaxsq(n_types,n_types), STAT=ierr)

      ! Statistical variables
      ALLOCATE(vol_stat(2), STAT=ierr)
      ALLOCATE(multimove_stat(2), STAT=ierr)

      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATA ERROR: ALLOCATION OF ARRAY FAILED IN SETUP_ALLOCATE SUBROUTINE'
        STOP
      ENDIF
  
    ! Block averages
    CASE('block')

      ! Allocate block average arrays
      ALLOCATE(blk_nmol(n_steps,n_types), STAT=ierr)
      ALLOCATE(blk_rho(n_steps,n_types), STAT=ierr)
      ALLOCATE(blk_eng(n_steps), STAT=ierr)
      ALLOCATE(blk_vir(n_steps), STAT=ierr)
      ALLOCATE(blk_vol(n_steps), STAT=ierr)
      ALLOCATE(avg_nmol(n_blocks,n_types), STAT=ierr)
      ALLOCATE(avg_rho(n_blocks,n_types), STAT=ierr)
      ALLOCATE(avg_eng(n_blocks), STAT=ierr)
      ALLOCATE(avg_vir(n_blocks), STAT=ierr)
      ALLOCATE(avg_vol(n_blocks), STAT=ierr)

      ! Initialize average block statistics
      avg_nmol = 0.0d0
      avg_rho = 0.0d0
      avg_eng = 0.0d0
      avg_vir = 0.0d0
      avg_vol = 0.0d0

      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATA ERROR: ALLOCATION OF ARRAY FAILED IN SETUP_ALLOCATE SUBROUTINE'
        STOP
      ENDIF

    ! Optional sampling variables
    CASE('optional')
    
      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATA ERROR: ALLOCATION OF ARRAY FAILED IN SETUP_ALLOCATE SUBROUTINE'
        STOP
      ENDIF


    CASE DEFAULT

      WRITE(*,*) 'FATAL ERROR: INVALID FLAG IN GLOBAL_ALLOCATE SUBROUTINE'
      STOP

  END SELECT


  RETURN

END SUBROUTINE global_allocate
