MODULE global


   ! Ensemble
   !=======================================================!
   INTEGER :: ensmbl


   ! System dimensions
   !=======================================================!

   ! Simulation box dimensions (A)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: box

   ! Simulation box volume (cu.A)
   DOUBLE PRECISION :: vol


   ! Molecular topology
   !=======================================================!

   ! Maximum number of molecules in each box
   INTEGER,PARAMETER :: n_mol_max = 2000

   ! Number of molecules types
   INTEGER :: n_types

   ! Maximum number of sites on each molecule
   INTEGER :: n_sites_max

   ! Number of molecules of each type
   INTEGER,DIMENSION(:),ALLOCATABLE :: n_mol

   ! Total number of molecules in each box
   INTEGER :: n_mol_tot

   ! Number of sites on each molecule
   INTEGER,DIMENSION(:),ALLOCATABLE :: n_sites

   ! Number of dispersion sites on each molecule
   INTEGER,DIMENSION(:),ALLOCATABLE :: n_disp

   ! Number of ion sites on each molecule
   INTEGER,DIMENSION(:),ALLOCATABLE :: n_ions

   ! Type of each molecule
   INTEGER,DIMENSION(:),ALLOCATABLE :: mol_type
  
    ! Name of each molecule type
   CHARACTER(LEN=4),DIMENSION(:),ALLOCATABLE :: mol_name

   ! Name of each molecule site
   CHARACTER(LEN=4),DIMENSION(:,:),ALLOCATABLE :: site_name

   ! List of dispersion sites on each molecule type
   INTEGER,DIMENSION(:,:),ALLOCATABLE :: disp_sites

   ! List of ion sites on each molecule type
   INTEGER,DIMENSION(:,:),ALLOCATABLE :: ion_sites 
 
      ! Mass of each site type
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: site_mass

   ! Mass of each molecule type
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: mol_mass

   ! Number of rotational degrees of freedom for each molecule type
   INTEGER,DIMENSION(:),ALLOCATABLE :: n_rot_dof

   ! Inertia tensor for each molecule and its inverse
   DOUBLE PRECISION,DIMENSION(:,:,:),ALLOCATABLE :: inrta_tnsr, inrta_tnsr_inv
   
   ! Center of mass positions of each molecule
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: rx, ry, rz

   ! Center of mass velocity of each molecule
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: vx, vy, vz
 
   ! Angular velocities of each molecule (body frame)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: wx, wy, wz
 
   ! Orientation quaternions of each molecule 
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: q0, q1, q2, q3

   ! Momentum quaternion of each molecule
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: p0, p1, p2, p3

   ! Position of the sites on each molecule
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: rx_s, ry_s, rz_s

   ! Internal coordinates of the sites on each molecule (wrt molecule's COM)
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: rx_i, ry_i, rz_i


   ! Potential parameters
   !=======================================================!

   ! Dispersion and electrostatic parameters
   DOUBLE PRECISION,DIMENSION(:,:,:,:),ALLOCATABLE :: eps,sig,alpha,q,sigsq,qsq,rmin,rminsq
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: rmax,rmaxsq


   ! Thermodynamic and mechanical properties
   !=======================================================!

   ! Temperature (K)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: temp

   ! Pressure (bar)
   DOUBLE PRECISION :: press 

   ! Energy (K)
   DOUBLE PRECISION :: energy

   ! Net force on each molecule (singe  array for x,y,z)
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: force

   ! Net torque on each molecule (singe array for x,y,z)
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: torque

   ! Shift to the Lennard Jones potential
   DOUBLE PRECISION :: lj_shift

   ! Long-range Lennard Jones factor
   DOUBLE PRECISION :: lrc_fact_npt


   ! Monte Carlo run parameters
   !=======================================================!

   ! Number of equilibration MC blocks
   INTEGER :: n_equil

   ! Number of MC blocks
   INTEGER :: n_blocks

   ! Number of MC steps per block
   INTEGER :: n_steps

   ! Current block number
   INTEGER :: iblock

   ! Current step number
   INTEGER :: istep

   ! Number of MD time steps
   INTEGER :: n_md_steps

   ! Time step for molecular dynamics
   DOUBLE PRECISION :: tstep

 
   ! Basic statistical variables
   !=======================================================!

   ! Move acceptance statistics
   INTEGER,DIMENSION(:),ALLOCATABLE :: vol_stat,multimove_stat

   ! Block statistics
   INTEGER,DIMENSION(:,:),ALLOCATABLE :: blk_nmol
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: blk_rho
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: blk_eng, blk_vir, blk_vol
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: avg_nmol,avg_rho
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: avg_eng, avg_vir, avg_vol   

   ! Maximum volume change
   DOUBLE PRECISION :: max_vol


   ! Optional variables
   !=======================================================!
   !=======================================================!
   
   ! Flag to turn on/off optional sampling
   INTEGER :: opt

   ! Trajectory output
   !=======================================================!

   ! Flag to turn on/off trajectory dumping (0 = off, 1= ASCII, 2 = binary)
   INTEGER :: dump_traj

   ! Number of MC steps between trajectory frame dumps
   INTEGER :: n_traj


   ! Ewald summation
   !=======================================================!

   ! Flag to turn on/off Ewald 
   INTEGER :: ewld

   ! Flag to tur on vacuum boundary conditions
   INTEGER :: ewld_vac

   ! Tolerance parameters
   DOUBLE PRECISION :: etol

   ! Convergence parameter
   DOUBLE PRECISION :: kappa

   ! Cutoff (squared) in reciprocal space
   DOUBLE PRECISION :: ksq_max

   ! Box dimensions for ewald (used for determining number of kvec ONLY!!!)
   DOUBLE PRECISION,DIMENSION(3) :: ewld_box

   ! Maximum number of reciprocal vectors
   INTEGER :: maxk
   INTEGER :: n_k
   INTEGER :: k_max_max
   INTEGER,DIMENSION(:),ALLOCATABLE :: k_max
   INTEGER,DIMENSION(:),ALLOCATABLE :: kvx,kvy,kvz

   ! Reciprocal vector array
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: k_vec

   ! Reciprocal component vectors (fluid molecules)
   COMPLEX*16,DIMENSION(:,:,:),ALLOCATABLE :: eikx, eiky, eikz

   ! Reciprocal component vectors for a single molecule
   COMPLEX*16,DIMENSION(:,:),ALLOCATABLE :: eikx_mol, eiky_mol, eikz_mol

   ! Reciprocal structure factor (fluid molecules)
   COMPLEX*16,DIMENSION(:),ALLOCATABLE :: skewld,skewld_new

   ! Self energy (fluid molecules)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: ewld_self

   ! Surface components (fluid molecules)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: ewld_surfc, ewld_surfc_new
  

   ! Weighted ensemble sampling
   !=======================================================!

   ! Flag to turn on/off weighted ensemble sampling
   INTEGER :: wensmbl

   ! Number of collective variable (CV) constraints
   INTEGER :: n_cvs
  
   ! Frequency (in steps) to dump CVs
   INTEGER :: n_cvdump
 
   ! Array specifying the type of each CV
   ! Value : 4 = density; 5 = Steinhardt Q6
   INTEGER,DIMENSION(:),ALLOCATABLE :: cv_types

   ! Flags for specifying the types of MC moves that each CV affects (0 = off, 1 = on)
   ! Index 1, move type: 1 = displacements; 2 = rotations; 3 = volume changes; 4 = particle deletions/insertions (including transfers)
   ! Index 2, CV type:   1 = density; 2 = Steinhardt Q6
   INTEGER,DIMENSION(:,:),ALLOCATABLE :: cv_moves

   ! Array for holding various parameters necessary for calculating CV
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: rho, krho, rhostar

   ! Array for holding various parameters necessary for calculating CV
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: Q6, kQ6, Q6star

   ! New values of the parameters used to compute change in energy
   DOUBLE PRECISION :: rho_new, Q6_new

   ! Array for holding processor numbers of each replica
   INTEGER,DIMENSION(:),ALLOCATABLE :: proclist
   DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: procparam

   ! Energy for each CV (K)
   DOUBLE PRECISION,DIMENSION(:),ALLOCATABLE :: cv_eng, cv_eng_new

   ! Review trajectory (analyze)
   !=======================================================!

   ! Flag to turn on/off trajectory review
   INTEGER :: traj

   ! Logical for xyz of dcd file format (0 = xyz, 1 = dcd)
   INTEGER :: xyz_dcd

   ! First frames in the trajectory file to be analyzed
   INTEGER :: first_frame
 
   ! Last frame in the trajectory file to be analyzed
   INTEGER :: last_frame

   ! Frequency of frames to be analyzed
   INTEGER :: frame_freq


   ! Random number generator
   !=======================================================!

   INTEGER :: idum
   DOUBLE PRECISION,EXTERNAL :: random


   ! MPI parameters
   !=======================================================!
   
   ! Total number processors
   INTEGER :: n_procs

   ! Rank of the current processor [0 to n_proc -1]
   INTEGER :: my_rank

   ! Current processor number (for looping)
   INTEGER :: i_proc

   ! Error handler (> 0 means error)
   INTEGER :: mpi_err
    

   ! Physical constants and conversion factors
   !=======================================================!

   DOUBLE PRECISION,PARAMETER :: Pi = 3.141592653589d0
   DOUBLE PRECISION,PARAMETER :: Kb = 1.3806504d-23   ! J/K
   DOUBLE PRECISION,PARAMETER :: Na = 6.02214179d23   ! 1/mol
   DOUBLE PRECISION,PARAMETER :: h = 6.62606896d-34   ! J/s
   DOUBLE PRECISION,PARAMETER :: R = Kb*Na            ! J/(mol K)
   DOUBLE PRECISION,PARAMETER :: Pp = Kb*1.0d30       ! (J cu.A)/(K cu.M) OR (Pa cu.A)/K
   DOUBLE PRECISION,PARAMETER :: Pb = Pp*1.0d-5       ! (bar cu. A)/K
   DOUBLE PRECISION,PARAMETER :: Pr = Na*1.0d-30
   DOUBLE PRECISION,PARAMETER :: Pe  = 167100.956613739d0 ! K Angstrom (for electrostatics)
   DOUBLE PRECISION,PARAMETER :: Fc = 0.1d0*R         ! (g A^2)/(mol K ps^2)


END MODULE global 
