SUBROUTINE comp_kin_eng(kin_eng,kin_temp)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  DOUBLE PRECISION :: kin_eng,kin_temp

  ! Local
  INTEGER :: imol,j
  DOUBLE PRECISION :: ker,ket 
  DOUBLE PRECISION :: trans_temp, rot_temp

  kin_eng = 0.0d0
  kin_temp = 0.0d0
  ket = 0.0d0
  ker = 0.0d0

  ! Compute translational kinetic energy and temperature
  DO imol = 1,n_mol_tot
    ket = ket + vx(imol)**2 + vy(imol)**2 + vz(imol)**2 ! [A^2/ps^2]
  ENDDO
  ket = 0.5d0*ket*mol_mass(1)/Fc ! [K]
  trans_temp = (2.0d0/(3.0d0*DBLE(n_mol_tot) - 3.0d0))*(ket) ! [T]

  ! Update the kinetic energy and temperature
  kin_eng = kin_eng + ket
  kin_temp = kin_temp + trans_temp
 
  ! Return if there are no rotational degrees of freedom
  IF(n_rot_dof(1) .EQ. 0) RETURN

  ! Compute rotational kinetic energy and temperature
  DO imol = 1,n_mol_tot
    ker = ker + inrta_tnsr(1,1,1)*wx(imol)**2 + inrta_tnsr(2,2,1)*wy(imol)**2 + inrta_tnsr(3,3,1)*wz(imol)**2  ! [(A^2 g) / (mol ps^2)]
  ENDDO
  ker = 0.5d0*ker/Fc  ![K]
  rot_temp = (2.0d0/(DBLE(n_rot_dof(1)*n_mol_tot)))*(ker)  ! [K]


  ! Compute the total kinetic energy and avg temperature
  kin_eng = kin_eng + ker
  kin_temp = 0.5d0*(kin_temp +  rot_temp)
  
  RETURN

  
END SUBROUTINE comp_kin_eng

