PROGRAM MC

  USE global
  IMPLICIT NONE
  include 'mpif.h'

  ! Local
  INTEGER :: ibox
  DOUBLE PRECISION :: engtst,  engerr
  DOUBLE PRECISION,DIMENSION(3,n_mol_max) :: frctst,trqtst

  ! Initialize iblock and istep
  iblock = 0
  istep = 0
 
  
  ! Read the input file
  CALL read_input

  ! Syncronize processors
  IF(n_procs .GT. 1) CALL MPI_BARRIER(MPI_COMM_WORLD, mpi_err)

  ! Open dump files for writing
  CALL dump(0)

  ! Calculate energy
  CALL engforce_total(energy,force,torque)

  ! Calculate CVs and CV energies
  IF(wensmbl .EQ. 1) CALL weighted_ensmbl(1,0,0,0,0)

  ! Write initial configuration
  CALL write_config(0)

  ! Synchronize processors
  IF(n_procs .GT. 1) CALL MPI_BARRIER(MPI_COMM_WORLD, mpi_err)

  ! Loop over the blocks
  DO iblock = 1,n_blocks

    ! Reset basic statistics 
    CALL stats(1)

    ! Loop over the steps per block
    DO istep = 1,n_steps
      
      ! Perform a trial move
      CALL trial_move
    
      ! Accumulate basic statistics
      CALL stats(2)

      ! Write dumps to file/screen
      CALL dump(2)

    ! End loop over the steps per block
    ENDDO

    ! Average basic statistics for iblock
    CALL stats(3)

    ! Write restart every 10 blocks
    IF(MOD(iblock,10) .EQ. 0) THEN
      CALL write_restart
    ENDIF

    ! Write configuration every 250 blocks
    IF(MOD(iblock,1000) .EQ. 0) CALL write_config

    ! Print status 
    IF(my_rank .EQ. 1) THEN
      IF(iblock .LE. n_equil) THEN
        IF(iblock .EQ. 1000) WRITE(*,'(A)')'Equilibration stage... '
        IF(MOD(NINT(100.0d0*DBLE(iblock)),10*n_equil) .EQ. 0) THEN
          WRITE(*,'(I0,A3)')NINT(100.0d0*DBLE(iblock)/DBLE(n_equil)),'%  '
        ENDIF
         IF(iblock .EQ. n_equil) WRITE(*,*)''
      ELSE
        IF(iblock .EQ. n_equil + 1)  WRITE(*,'(A)')'Production stage... ' 
        IF(MOD(NINT(100.0d0*DBLE(iblock-n_equil)),10*(n_blocks-n_equil)).EQ. 0) THEN
          WRITE(*,'(I0,A3)')NINT(100.0d0*DBLE(iblock-n_equil)/DBLE(n_blocks-n_equil)),'%  '
        ENDIF
        IF(iblock .EQ. n_blocks) WRITE(*,*)'' 
      ENDIF

      ! Check the total and updated energy consistency for each box (every 100 blocks)
 !     IF(MOD(iblock,NINT(DBLE(n_blocks)/10.0d0)+1) .EQ. 0) THEN
 !       CALL engforce_total(engtst,frctst,trqtst)
 !       IF(engtst .NE. 0.0d0) THEN
 !         engerr = dABS((engtst - energy)/engtst)
 !       ELSE
 !         engerr = dABS((engtst - energy))
 !       ENDIF
 !       IF (engerr .GE. 1.0d-5) THEN
 !         WRITE(*,*) 'WARNING: LARGE ERROR BETWEEN THE TOTAL AND UPDATED ENERGY'
 !         WRITE(*,'(A7,I0,A10,F20.8,A10,F20.8)')&
 !         &'BLOCK: ',iblock,' UPD ENG: ',energy,' TOT ENG: ',engtst
 !       ENDIF
 !     ENDIF
    ENDIF

  

  ! End loop over the blocks
  ENDDO

  ! Write stats to file
  CALL stats(4)

  ! Set iblcok to n_blocks + 1
  iblock = n_blocks + 1

  ! Close dump files
  CALL dump(4)
  
  ! Write the final configurations
  CALL write_config

  ! Close MPI
  IF(n_procs .GT. 1) CALL MPI_FINALIZE(mpi_err)


END PROGRAM MC
