SUBROUTINE md_move

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Local
  INTEGER :: imol,i,itype,jtype,ierr,jstep
  DOUBLE PRECISION :: ket_new, ket_old, kin_temp_new, kin_temp_old
  DOUBLE PRECISION :: eng_new, eng_old, eng_dum
  DOUBLE PRECISION :: eng_chng
  DOUBLE PRECISION :: ham_old,ham_new
  DOUBLE PRECISION :: ran,boltzf
  DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: rcom_old
  DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: q_old
  DOUBLE PRECISION,DIMENSION(:,:,:),ALLOCATABLE :: rs_old
  DOUBLE PRECISION,DIMENSION(:,:),ALLOCATABLE :: force_old, torque_old
  DOUBLE PRECISION,DIMENSION(3) :: cv_data
  DOUBLE PRECISION :: cv_eng_chng
  LOGICAL :: ACCPT


  ! Assume the move is rejected
  ACCPT = .FALSE.

  ! Initialize the energy change to zero
  eng_chng = 0.0d0

  ! Initialize the CV energy change to zero
  cv_eng_chng = 0.0d0


  ! Store the old coordinates

  ! Allocate temporary arrays
  AlLOCATE(rcom_old(3,n_mol_max), STAT=ierr)
  AlLOCATE(q_old(4,n_mol_max), STAT=ierr)
  ALLOCATE(rs_old(3,n_sites_max,n_mol_max), STAT=ierr)
  ALLOCATE(force_old(3,n_mol_max), STAT=ierr)
  ALLOCATE(torque_old(3,n_mol_max),STAT=ierr)

  ! Store the old COM positions
  rcom_old(1,:) = rx(:)
  rcom_old(2,:) = ry(:)
  rcom_old(3,:) = rz(:)

  ! Store the old quaternions
  q_old(1,:) = q0(:)
  q_old(2,:) = q1(:)
  q_old(3,:) = q2(:)
  q_old(4,:) = q3(:)

  ! Store the old site positions
  rs_old(1,:,:) = rx_s(:,:)
  rs_old(2,:,:) = ry_s(:,:)
  rs_old(3,:,:) = rz_s(:,:)
 
  ! Store the old force and torque
  eng_old = energy
  force_old = force
  torque_old = torque

  ! Update the number of multimove trials
  multimove_stat(1) = multimove_stat(1) + 1

  ! Initialize velocities
  CALL set_velocity
 
  ! Remove the center of mass velocity
  CALL remove_vcom
 
  !Calculate KE and PE in K
  CALL comp_kin_eng(ket_old,kin_temp_old)

  ! Perform translation steps
  DO  jstep = 1,n_md_steps

    ! Perform first half of the velocity verlet integration
    CALL nveqvv_1(1)


    ! Calculate the total energy for each box (Note: Includes Ewald already)
    CALL engforce_total(eng_new,force,torque)

    ! Perform second half of the velocity verlet integration
    CALL nveqvv_1(2)

    ! Remove the center of mass velocity
    CALL remove_vcom

  ENDDO


  !Calculate KE and PE in K
  CALL  comp_kin_eng(ket_new,kin_temp_new)

  ! Calculate the energy change
  eng_chng =  (ket_new - ket_old) + (eng_new - eng_old)



  ! If running weighted ensemble simulation
  IF(wensmbl .EQ. 1) THEN

    ! Calculate the weights
    CALL weighted_ensmbl(2,1,imol,cv_data,cv_eng_chng)

  ENDIF

  ! Check for large energies to avoid underflow
  IF((eng_chng + cv_eng_chng)/temp(my_rank) .GT. 180.0d0) GOTO 10

  ! Apply the Metropolis criteria
  IF((eng_chng + cv_eng_chng) .LT. 0.0d0) THEN
    ACCPT = .TRUE.
  ELSE
    boltzf = MIN(1.0d0,dEXP(-(eng_chng + cv_eng_chng)/temp(my_rank)))
    IF(random(idum) .LT. boltzf) ACCPT = .TRUE.
  ENDIF


  ! Continue here if overlap or large energy
10 CONTINUE

 ! WRITE(*,'(5F28.8,L4,I5)') (ket_new - ket_old)+ (eng_new - eng_old),kin_temp_new, kin_temp_old, eng_new,eng_old, ACCPT, n_md_steps


  ! If the move is accepted
  IF(ACCPT) THEN

    ! Update the number of multimove trials
    multimove_stat(2) = multimove_stat(2) + 1

    ! Update the energy 
    energy = eng_new 

    ! Update the CVs and CV energies
    IF(wensmbl .EQ. 1) CALL weighted_ensmbl(3,1,imol,cv_data,0.0d0)

  ELSE

   ! Reset the total energy
   energy = eng_old

   ! Reset the old COM positions
   rx(:) =  rcom_old(1,:) 
   ry(:) =  rcom_old(2,:) 
   rz(:) =  rcom_old(3,:) 

   ! Reset the old quaternions
   q0(:) = q_old(1,:)
   q1(:) = q_old(2,:) 
   q2(:) = q_old(3,:) 
   q3(:) = q_old(4,:) 

   ! Reset the old site positions
   rx_s(:,:) = rs_old(1,:,:) 
   ry_s(:,:) = rs_old(2,:,:) 
   rz_s(:,:) = rs_old(3,:,:)

   ! Reset the old force and torque
   force = force_old
   torque = torque_old

  ENDIF

  ! Deallocate position arrays
  DEALLOCATE(rcom_old,q_old,rs_old,force_old,torque_old)


  RETURN

END SUBROUTINE md_move
