SUBROUTINE nosquish(imol,qq0,qq1,qq2,qq3,pp0,pp1,pp2,pp3)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: imol

  ! Local
  INTEGER ::  m
  DOUBLE PRECISION :: zetax,zetay,zetaz,cs,sn,trstep
  DOUBLE PRECISION :: qn1(0:3),pq2(0:3),qn2(0:3),pq3(0:3)
  DOUBLE PRECISION :: qn3(0:3),pq4(0:3)
  DOUBLE PRECISION :: qq0(*), qq1(*),qq2(*),qq3(*)
  DOUBLE PRECISION :: pp0(*), pp1(*),pp2(*),pp3(*)
  INTEGER, PARAMETER :: mrot=1
  DOUBLE PRECISION, PARAMETER :: ov4=0.25d0
  DOUBLE PRECISION, PARAMETER :: ov8=0.125d0


  !Calculate the rotational time step
  trstep=tstep/DBLE(mrot)
 
  ! Perform rotations: iterate over mrot rotational time steps

  DO m=1,mrot

    zetaz = ov8*trstep*(-pp0(imol)*qq3(imol) + pp1(imol)*qq2(imol) - pp2(imol)*qq1(imol) + pp3(imol)*qq0(imol))*inrta_tnsr_inv(3,3,1)
    cs = dCOS(zetaz)
    sn = dSIN(zetaz)
    qn1(0) = cs*qq0(imol) - sn*qq3(imol)
    qn1(1) = cs*qq1(imol) + sn*qq2(imol)
    qn1(2) = cs*qq2(imol) - sn*qq1(imol)
    qn1(3) = cs*qq3(imol) + sn*qq0(imol)
    pq2(0) = cs*pp0(imol) - sn*pp3(imol)
    pq2(1) = cs*pp1(imol) + sn*pp2(imol)
    pq2(2) = cs*pp2(imol) - sn*pp1(imol)
    pq2(3) = cs*pp3(imol) + sn*pp0(imol)

    zetay = ov8*trstep*(-pq2(0)*qn1(2) - pq2(1)*qn1(3) + pq2(2)*qn1(0) + pq2(3)*qn1(1))*inrta_tnsr_inv(2,2,1)
    cs = dCOS(zetay)
    sn = dSIN(zetay)
    qn2(0) = cs*qn1(0) - sn*qn1(2)
    qn2(1) = cs*qn1(1) - sn*qn1(3)
    qn2(2) = cs*qn1(2) + sn*qn1(0)
    qn2(3) = cs*qn1(3) + sn*qn1(1)
    pq3(0) = cs*pq2(0) - sn*pq2(2)
    pq3(1) = cs*pq2(1) - sn*pq2(3)
    pq3(2) = cs*pq2(2) + sn*pq2(0)
    pq3(3) = cs*pq2(3) + sn*pq2(1)
        
    zetax = ov4*trstep*(-pq3(0)*qn2(1) + pq3(1)*qn2(0) + pq3(2)*qn2(3) - pq3(3)*qn2(2))*inrta_tnsr_inv(1,1,1)
    cs = dCOS(zetax)
    sn = dSIN(zetax)
    qn3(0) = cs*qn2(0) - sn*qn2(1)
    qn3(1) = cs*qn2(1) + sn*qn2(0)
    qn3(2) = cs*qn2(2) + sn*qn2(3)
    qn3(3) = cs*qn2(3) - sn*qn2(2)
    pq4(0) = cs*pq3(0) - sn*pq3(1)
    pq4(1) = cs*pq3(1) + sn*pq3(0)
    pq4(2) = cs*pq3(2) + sn*pq3(3)
    pq4(3) = cs*pq3(3) - sn*pq3(2)
        
    zetay = ov8*trstep*(-pq4(0)*qn3(2) - pq4(1)*qn3(3) + pq4(2)*qn3(0) + pq4(3)*qn3(1))*inrta_tnsr_inv(2,2,1)
    cs = dCOS(zetay)
    sn = dSIN(zetay)
    qn2(0) = cs*qn3(0) - sn*qn3(2)
    qn2(1) = cs*qn3(1) - sn*qn3(3)
    qn2(2) = cs*qn3(2) + sn*qn3(0)
    qn2(3) = cs*qn3(3) + sn*qn3(1)
    pq3(0) = cs*pq4(0) - sn*pq4(2)
    pq3(1) = cs*pq4(1) - sn*pq4(3)
    pq3(2) = cs*pq4(2) + sn*pq4(0)
    pq3(3) = cs*pq4(3) + sn*pq4(1)
    
    zetaz = ov8*trstep*(-pq3(0)*qn2(3) + pq3(1)*qn2(2) - pq3(2)*qn2(1) + pq3(3)*qn2(0))*inrta_tnsr_inv(3,3,1)
    cs = dCOS(zetaz)
    sn = dSIN(zetaz)
    qq0(imol) = cs*qn2(0) - sn*qn2(3)
    qq1(imol) = cs*qn2(1) + sn*qn2(2)
    qq2(imol) = cs*qn2(2) - sn*qn2(1)
    qq3(imol) = cs*qn2(3) + sn*qn2(0)
    pp0(imol) = cs*pq3(0) - sn*pq3(3)
    pp1(imol) = cs*pq3(1) + sn*pq3(2)
    pp2(imol) = cs*pq3(2) - sn*pq3(1)
    pp3(imol) = cs*pq3(3) + sn*pq3(0)
 
       
  ENDDO

  RETURN
  
END SUBROUTINE nosquish



