SUBROUTINE nveqvv_1(fstep)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: fstep

  INTEGER :: imol, isite
  DOUBLE PRECISION :: half_step
  DOUBLE PRECISION :: opx,opy,opz
  DOUBLE PRECISION :: tx,ty,tz
  DOUBLE PRECISION :: g0,g1,g2,g3
  DOUBLE PRECISION,DIMENSION(3) :: invec, outvec
  DOUBLE PRECISION,DIMENSION(4) :: quat
  DOUBLE PRECISION,DIMENSION(3,3) :: rotmat


  ! Compute quaternion momenta from angular velocities (initialization step)
  IF(fstep .EQ. 1) THEN

    DO imol = 1,n_mol_tot

      ! Compute quaternion momenta from angular velocities
      opx = wx(imol)*inrta_tnsr(1,1,1)  ![(A^2 g)/(mol ps)]
      opy = wy(imol)*inrta_tnsr(2,2,1)
      opz = wz(imol)*inrta_tnsr(3,3,1)

      p0(imol) = 2.0d0*(-q1(imol)*opx - q2(imol)*opy - q3(imol)*opz) ![(A^2 g)/(mol ps)]
      p1(imol) = 2.0d0*( q0(imol)*opx - q3(imol)*opy + q2(imol)*opz)
      p2(imol) = 2.0d0*( q3(imol)*opx + q0(imol)*opy - q1(imol)*opz)
      p3(imol) = 2.0d0*(-q2(imol)*opx + q1(imol)*opy + q0(imol)*opz)

    ENDDO

  ENDIF
 
  
  ! Poritions common to each half step
  DO imol = 1,n_mol_tot

    ! Compute half the timestep
    half_step = 0.5d0*tstep ! (ps)

    ! Translation 

    ! Update center of mass velocities by 1/2 timestep
    vx(imol) = vx(imol) + half_step*force(1,imol)/mol_mass(1) ! [A/ps] = [A/ps] + [ps] * [(g A)/(mol ps^2)] * [mol/g]
    vy(imol) = vy(imol) + half_step*force(2,imol)/mol_mass(1)
    vz(imol) = vz(imol) + half_step*force(3,imol)/mol_mass(1)
  
    ! Rotation

    
    ! Create quaternion array
    quat(1) = q0(imol)
    quat(2) = q1(imol)
    quat(3) = q2(imol)
    quat(4) = q3(imol)

    ! Get the rotation matrix to convert from space frame to body frame
    CALL rot_matrix(1,quat,rotmat)

    ! Compute torque in body-centered frame [(g A^2)/(mol ps^2)]
    tx = torque(1,imol)*rotmat(1,1) + torque(2,imol)*rotmat(2,1) + torque(3,imol)*rotmat(3,1)
    ty = torque(1,imol)*rotmat(1,2) + torque(2,imol)*rotmat(2,2) + torque(3,imol)*rotmat(3,2)
    tz = torque(1,imol)*rotmat(1,3) + torque(2,imol)*rotmat(2,3) + torque(3,imol)*rotmat(3,3)    
 
    ! Compute quaternion torques [(g A^2)/(mol ps^2)]
    g0 = 2.0d0*(-q1(imol)*tx - q2(imol)*ty - q3(imol)*tz)
    g1 = 2.0d0*( q0(imol)*tx - q3(imol)*ty + q2(imol)*tz)
    g2 = 2.0d0*( q3(imol)*tx + q0(imol)*ty - q1(imol)*tz)
    g3 = 2.0d0*(-q2(imol)*tx + q1(imol)*ty + q0(imol)*tz)
 
    ! Update the quaternion momenta ![(A^2 g)/(mol ps)] = [ps] * [(A^2 g)/(mol ps^2)]
    p0(imol) = p0(imol) + half_step*g0 
    p1(imol) = p1(imol) + half_step*g1
    p2(imol) = p2(imol) + half_step*g2
    p3(imol) = p3(imol) + half_step*g3

  ENDDO


  
  ! If this is the first half step
  IF(fstep .EQ. 1) THEN

    ! Translation

    ! Loop over the molecules
    DO imol = 1 , n_mol_tot
 
      ! Update center of mass positions by full timestep
      rx(imol) = rx(imol) + tstep*vx(imol) ! [A] = [A] + [ps] * [A/ps]
      ry(imol) = ry(imol) + tstep*vy(imol)
      rz(imol) = rz(imol) + tstep*vz(imol)
    ENDDO     


    ! Rotation

   ! Loop over the molecules
    DO imol = 1, n_mol_tot
      ! Rotate the molecules with nosquish
      CALL nosquish(imol,q0,q1,q2,q3, p0,p1,p2,p3)
    ENDDO


    ! Loop over the molecules
    DO imol = 1,n_mol_tot

      ! Apply periodic boundaries to the new positions
      rx(imol) = rx(imol) - dNINT(rx(imol)/box(1))*box(1)
      ry(imol) = ry(imol) - dNINT(ry(imol)/box(2))*box(2)
      rz(imol) = rz(imol) - dNINT(rz(imol)/box(3))*box(3)

      ! Calculate the atomic site positions after displacement of COM and quaternion update
      CALL site_coord(imol,q0(imol),q1(imol),q2(imol),q3(imol))
      
    ENDDO

  ELSE

    ! Loop over the molecules
    DO imol = 1,n_mol_tot
      ! Calculate new angular velocities (boy frame)
      wx(imol) = 0.50d0*(-q1(imol)*p0(imol) + q0(imol)*p1(imol) + q3(imol)*p2(imol) - q2(imol)*p3(imol))*inrta_tnsr_inv(1,1,1) ! [1/ps] = [(gA^2)/(mol ps)] * [mol/g A^2]
      wy(imol) = 0.50d0*(-q2(imol)*p0(imol) - q3(imol)*p1(imol) + q0(imol)*p2(imol) + q1(imol)*p3(imol))*inrta_tnsr_inv(2,2,1)
      wz(imol) = 0.50d0*(-q3(imol)*p0(imol) + q2(imol)*p1(imol) - q1(imol)*p2(imol) + q0(imol)*p3(imol))*inrta_tnsr_inv(3,3,1)
    ENDDO

    ! Loop over the molecules
    DO imol = 1,n_mol_tot

      ! Apply periodic boundaries to the new positions
      rx(imol) = rx(imol) - dNINT(rx(imol)/box(1))*box(1)
      ry(imol) = ry(imol) - dNINT(ry(imol)/box(2))*box(2)
      rz(imol) = rz(imol) - dNINT(rz(imol)/box(3))*box(3)

      ! Calculate the atomic site positions after displacement of COM and quaternion update
      CALL site_coord(imol,q0(imol),q1(imol),q2(imol),q3(imol))

    ENDDO


  ENDIF

  RETURN
  
END SUBROUTINE nveqvv_1



