SUBROUTINE set_ewld(selector)

  USE global
  IMPLICIT NONE 
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: selector

  ! Local
  INTEGER :: kx,ky,kz,k
  INTEGER :: imol,itype,isite,ichrg
  INTEGER :: n_mol_tot_temp
  INTEGER,DIMENSION(n_types) :: n_mol_temp
  DOUBLE PRECISION :: rksq,rkx,rky,rkz
  DOUBLE PRECISION :: ksq,b,maxbox
  DOUBLE PRECISION, PARAMETER :: two_Pi = 2.0d0*Pi
  DOUBLE PRECISION,DIMENSION(3) :: two_Pibox
  DOUBLE PRECISION :: ewldeng_temp
  INTEGER :: ierr


  ! Select a case
  SELECT CASE (selector)

    ! Perform one time allocation and setup tasks
    CASE(0)

      ! Initialize the error handler
      ierr = 0

      ! Allocate the arrays
      ALLOCATE(k_max(3), STAT=ierr)
      ALLOCATE(ewld_self(n_types), STAT=ierr)
      ALLOCATE(ewld_surfc(3), STAT=ierr)
 
      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATAL ERROR: ALLOCATION OF ARRAYS FAILED IN SUBROUTINE SET_EWLD (1)'
        STOP
      ENDIF


      ! Calculate corresponding convergence parameter using minimum cutoff distance
      kappa = (1.35d0 - 0.15d0*dLOG(etol))/MINVAL(rmax)

      ! Compute the self energy contribution for each type of molecule
      CALL set_ewldself

       WRITE(*,'(A,E12.2,A,F15.8)') 'Using Ewald summation with tolerance : ',etol, ' and Kappa : ',kappa


      ! Calculate kmax (rounding up)
      maxbox = MAXVAL(ewld_box(:))
      k_max(:) = INT((kappa*maxbox/Pi)*dSQRT(-dLOG(etol)))
      k_max_max = MAXVAL(k_max(:)) + 3  ! Upper array limit to avoid re-allocation

      ! Calculate maxk (upper array limit)
      maxk = 3*(MAXVAL(k_max(:))+3)**3

      ! Calculate ksq_max
      ksq_max = -4.0d0*kappa*kappa*dLOG(etol)

       !DBLE(CEILING(1.05d0*MAXVAL(k_max(:))**2)) * ((two_Pi/MAXVAL(box(:)))**2)

      ! Print k_max to screen
       WRITE(*,'(A,3I3,A,F15.8)') 'k-vectors (x,y,z): ',k_max(1),k_max(2),k_max(3), ' ksq max: ', ksq_max



      ! Initialize the error handler
      ierr = 0

      ! Allocate the arrays
      ALLOCATE(k_vec(maxk), STAT=ierr)
      ALLOCATE(kvx(maxk), STAT=ierr)
      ALLOCATE(kvy(maxk), STAT=ierr)
      ALLOCATE(kvz(maxk), STAT=ierr)
      ALLOCATE(eikx(MAXVAL(n_ions),n_mol_max,0:k_max_max), STAT=ierr)
      ALLOCATE(eiky(MAXVAL(n_ions),n_mol_max,-k_max_max:k_max_max), STAT=ierr)
      ALLOCATE(eikz(MAXVAL(n_ions),n_mol_max,-k_max_max:k_max_max), STAT=ierr)
      ALLOCATE(skewld(maxk), STAT=ierr)

      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATAL ERROR: ALLOCATION OF ARRAYS FAILED IN SUBROUTINE SET_EWLD (2)'
        STOP
      ENDIF


      ! Calculate the number of actual kvectors in the cutoff
      n_k = 0
      kvx = 0
      kvy = 0
      kvz = 0

 
      ! Calculate the reciprocal box vector
      two_Pibox(:) = two_Pi/box(:)

      DO kx = 0,k_max(1)
        DO ky = -k_max(2),k_max(2)
          DO kz = -k_max(3),k_max(3)

            ! Calculate the square of the reduced vector
            ksq = (DBLE(kx)*two_Pibox(1))**2 + (DBLE(ky)*two_Pibox(2))**2 + (DBLE(kz)*two_Pibox(3))**2

            ! Check the bounds of the vector
            IF((ksq .LT. ksq_max) .AND. (ABS(ksq) .GT. 1.0d-6)) THEN
              n_k = n_k + 1
              kvx(n_k) = kx
              kvy(n_k) = ky
              kvz(n_k) = kz    
            ENDIF

          ENDDO
        ENDDO
      ENDDO
      
      !WRITE(*,'(A80)')'========================================'&
      !            &//'========================================'

    ! Set/rest 
    CASE(1)
     

! Uncomment to allow kmax to change with box dimensions
      ! Calculate kmax (rounding up)
!      maxbox = MAXVAL(box(:))
!      k_max(:) = INT((kappa*maxbox/Pi)*dSQRT(-dLOG(etol)))


      ! Calculate the number of actual kvectors in the cutoff
!      n_k = 0
!      kvx = 0
!      kvy = 0
!      kvz = 0

      ! Calcualte the reciprocal box vector
!      two_Pibox(:) = two_Pi/box(:)

!      DO kx = 0,k_max(1)
!        DO ky = -k_max(2),k_max(2)
!          DO kz = -k_max(3),k_max(3)

            ! Calculate the square of the reduced vector
!            ksq = (DBLE(kx)*two_Pibox(1))**2 + (DBLE(ky)*two_Pibox(2))**2 + (DBLE(kz)*two_Pibox(3))**2

            ! Check the bounds of the vector
!            IF((ksq .LT. ksq_max) .AND. (ABS(ksq) .GT. 1.0d-6)) THEN
!              n_k = n_k + 1
!              kvx(n_k) = kx
!              kvy(n_k) = ky
!              kvz(n_k) = kz
!            ENDIF

!          ENDDO
!        ENDDO
!      ENDDO
 
      ! Setup calculate reciprocal vectors
 
      ! Calculate the denominator for the k-space exponential
      b = (1.0d0/(4.0d0*kappa**2))
 
      ! Loop over all the vectors
      DO k = 1,n_k

        ! Get kvector
        kx = kvx(k)
        ky = kvy(k)
        kz = kvz(k)

        ! Calculate rk components        
        rkx = two_Pi*DBLE(kx)/box(1)
        rky = two_Pi*DBLE(ky)/box(2)
        rkz = two_Pi*DBLE(kz)/box(3)

        ! Calculate rksq
        rksq = rkx**2 + rky**2 + rkz**2

        ! Calculate the the k-vector [units inverse length (1/A)]
        IF (kx .GT. 0) THEN
          k_vec(k) = 2.0d0*two_Pi*(1.0d0/vol)*dEXP(-b*rksq)/rksq
        ELSE
          k_vec(k) = two_Pi*(1.0d0/vol)*dEXP(-b*rksq)/rksq
        ENDIF

      ENDDO
     
 
    CASE DEFAULT

      WRITE(*,*) 'FATAL ERROR: INVALID FLAG TYPE IN SET_EWALD SUBROUTINE'
      STOP


  END SELECT

  RETURN

END SUBROUTINE set_ewld
