SUBROUTINE set_ewldself

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Local
  INTEGER :: itype,ichrg,jchrg,isite,jsite
  DOUBLE PRECISION :: rijs, rxijs, ryijs, rzijs
  DOUBLE PRECISION :: rija,rxija,ryija,rzija
  DOUBLE PRECISION :: f_factor

 
  ! Initialize the energy array
  ewld_self = 0.0d0

  ! Calculate the self energy for each fluid molecule type

  ! Calculate the self energy contribution from the same molecular site

  ! Loop over the molecule types
  DO itype = 1,n_types

    ! Loop over the charges on the molecule
    DO ichrg = 1,n_ions(itype)
     
      ! Get the site number of the charge
      isite = ion_sites(ichrg,itype)

      ! Calculate the self interaction
      ewld_self(itype) = ewld_self(itype) + q(isite,isite,itype,itype)*q(isite,isite,itype,itype)

    ENDDO

    ! Apply the prefactor
    ewld_self(itype) = kappa*ewld_self(itype)/dSQRT(Pi)
    
    ! Calculate the self energy contribution from other sites on the same molecule
      
    ! Loop over all charge pairs
    DO ichrg = 1,n_ions(itype) - 1
      isite = ion_sites(ichrg,itype)
      DO jchrg = ichrg + 1, n_ions(itype) 
        jsite = ion_sites(jchrg,itype)
       
        ! Calculate the separation vector between the two sites
        rxijs = rx_i(jsite,itype) - rx_i(isite,itype)
        ryijs = ry_i(jsite,itype) - ry_i(isite,itype)
        rzijs = rz_i(jsite,itype) - rz_i(isite,itype)

        ! Calculate scalar separation distance
        rijs = dSQRT(rxijs**2 + ryijs**2 + rzijs**2)
      
        ! Calculate the self interaction
        ewld_self(itype) = ewld_self(itype) + q(isite,isite,itype,itype)*q(jsite,jsite,itype,itype)*dERF(kappa*rijs)/rijs

      ENDDO
    ENDDO
  ENDDO

  RETURN

END SUBROUTINE set_ewldself
