SUBROUTINE set_initconf(filename)
  ! OPT

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  CHARACTER(LEN=*) :: filename

  ! Local
  INTEGER :: imol,jmol,itype,jtype,isite,n_total
  DOUBLE PRECISION :: rxi, ryi, rzi
  DOUBLE PRECISION :: q0i, q1i, q2i, q3i
  DOUBLE PRECISION,DIMENSION(2) :: engij,virij
  CHARACTER(LEN=10) :: temp_name



   WRITE(*,'(A,I0,3A)') 'Reading replica ', my_rank,' congifuration from ', TRIM(filename),' file'

   OPEN(UNIT=20,FILE=filename,STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

        ! Read the number of molecules
        READ(20,*) n_mol_tot
        n_mol(1) = n_mol_tot ! Hardcoded

        ! Read the box dimensions
        READ(20,*) box(1), box(2), box(3)

        ! Loop over the molecules
        DO imol = 1,n_mol_tot

          ! Read the type and coordinates to file
          READ(20,*) temp_name,rx(imol),ry(imol),rz(imol),&
                     &q0(imol),q1(imol),q2(imol),q3(imol) 
          
          ! Check that name length
          IF(LEN(TRIM(temp_name)) .GT. 4) THEN
             WRITE(*,*) 'FATAL ERROR: MOLECULE NAME: ',temp_name,' IN ',filename,' IS MORE THAN 4 CHARACTERS'
             STOP
          ENDIF
 
           ! Initialize the molecule's type
           mol_type(imol) = 0

           ! Set the molecule's type
           DO itype = 1,n_types
             IF(TRIM(temp_name) .EQ. mol_name(itype)) mol_type(imol) = itype
           ENDDO
          
           ! Compute the coordinates of the sites on the molecule
           CALL site_coord(imol,q0(imol),q1(imol),q2(imol),q3(imol))
             
           ! Throw and error if the type was not found
           IF(mol_type(imol) .EQ. 0) THEN
             WRITE(*,*) 'FATAL ERROR: NO MOLECULE PARAMETERS WERE FROUND FOR: ',temp_name,' IN ',filename
             STOP
           ENDIF

        ENDDO
 
  CLOSE(UNIT=20)


  RETURN

END SUBROUTINE set_initconf

