SUBROUTINE set_param(selector)
  !OPT
  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  CHARACTER(len=*) :: selector

  ! Local
  INTEGER :: i
  INTEGER :: itype, jtype
  INTEGER :: isite,jsite
  INTEGER :: ierr


  ! Select parameter group to initialize
  SELECT CASE (selector)

    ! Initialize system parameters
    CASE('system')

      ! Initialize thermodynamic variables
      energy = 0.0d0

      ! Initialize acceptance statistics
      vol_stat = 0
      multimove_stat = 0

      ! Initialize the total number of molecules in the boxes
      n_mol_tot = 0

      ! Initialize the number of collective variables
      n_cvs = 0

      ! Calculate the total number of molecules in each box
      DO itype = 1,n_types
        n_mol_tot = n_mol_tot + n_mol(itype)
      ENDDO

      ! Set the box volumes
      vol = box(1)*box(2)*box(3)
         
      ! Set flags for optional dumps (default = 0 (off))
      dump_traj = 0

      ! Set optional sampling (default = 0 (off))
      opt = 0     

      ! Initialize LRC factor
      lrc_fact_npt  = 0.0d0

      ! Initialize flags
      ewld = 0
      ewld_vac = 0
      wensmbl = 0
      traj = 0


    ! Initialize potential parameters
    CASE('potential')

      DO itype = 1,n_types
        DO jtype = 1,n_types

          ! Interaction cutoffs
          rmax(itype,jtype) = 0.50d0*(rmax(itype,itype)+rmax(jtype,jtype))
          rmaxsq(itype,jtype) = rmax(itype,jtype)**2

          DO isite = 1,n_sites(itype)
            DO jsite = 1,n_sites(jtype)

              ! Calculate interaction parameters using mixing rule
              sig(jsite,isite,jtype,itype) = 0.5d0*(sig(isite,isite,itype,itype)+sig(jsite,jsite,jtype,jtype))
              eps(jsite,isite,jtype,itype) = dSQRT(eps(isite,isite,itype,itype)*eps(jsite,jsite,jtype,jtype))
              alpha(jsite,isite,jtype,itype) = dSQRT(alpha(isite,isite,itype,itype)*alpha(jsite,jsite,jtype,jtype))
              rmin(jsite,isite,jtype,itype) = MAX(0.70d0*sig(jsite,isite,jtype,itype), 1.0d0)
              qsq(jsite,isite,jtype,itype) = q(isite,isite,itype,itype)*q(jsite,jsite,jtype,jtype)
              sigsq(jsite,isite,jtype,itype) = sig(jsite,isite,jtype,itype)**2
              rminsq(jsite,isite,jtype,itype) = rmin(jsite,isite,jtype,itype)**2

            ENDDO
          ENDDO

        ENDDO
      ENDDO


    CASE DEFAULT

      WRITE(*,*) 'FATAL ERROR: INVALID FLAG IN SET_PARAM SUBROUTINE'
      STOP

  END SELECT


  RETURN

END SUBROUTINE set_param
