SUBROUTINE set_velocity

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  INTEGER :: imol,j
  DOUBLE PRECISION :: sumvsq,  v_stdev   
  DOUBLE PRECISION :: sumwsq
  DOUBLE PRECISION,DIMENSION(3) ::w, w_stdev
  DOUBLE PRECISION :: ran_norm
  DOUBLE PRECISION :: trans_temp, rot_temp

  ! Routine only works with one molecule type
  IF(n_types .GT. 1) THEN
    WRITE(*,*) 'FATAL ERROR: MD ONLY WORKS FOR A SINGLE MOLECULE TYPE'
    STOP
  ENDIF

  ! Initialize the translational and angular velocities to zero
  vx = 0.0d0
  vy = 0.0d0
  vz = 0.0d0
  wx = 0.0d0
  wy = 0.0d0
  wz = 0.0d0

  ! Total kinetic energy is divided evenly between translational and rotational degrees of freedom

  ! Initialize translational velocities 
  
  ! Calculate the standard deviation of the Maxwell-Boltzmann distribution [SQRT(K)]
  v_stdev = dSQRT(temp(my_rank))
 
  DO imol = 1,n_mol_tot
   
    ! Generate velocities from the  Maxwell-Boltzmann distribution [SQRT(K)]
    vx(imol) = ran_norm(0.0d0,v_stdev)
    vy(imol) = ran_norm(0.0d0,v_stdev)
    vz(imol) = ran_norm(0.0d0,v_stdev)

  ENDDO
 
  ! Sum the velocity squared
  sumvsq = 0.0d0
  DO imol = 1,n_mol_tot
    sumvsq = sumvsq + vx(imol)**2 + vy(imol)**2 + vz(imol)**2  ! [K]
  ENDDO

  !  velocities [A/ps]
  vx = vx*dSQRT(Fc/mol_mass(1))
  vy = vy*dSQRT(Fc/mol_mass(1))
  vz = vz*dSQRT(Fc/mol_mass(1))


  ! Initialize rotational velocities

  ! Make sure that there are rotational degrees
  IF(n_rot_dof(1) .EQ. 0) RETURN

  ! Calculate the standard deviation of the Maxwell-Boltzmann distribution [SQRT[(K mol) /(A^2 g)]]
  w_stdev(1) = dSQRT(temp(my_rank)*inrta_tnsr_inv(1,1,1))
  w_stdev(2) = dSQRT(temp(my_rank)*inrta_tnsr_inv(2,2,1))
  w_stdev(3) = dSQRT(temp(my_rank)*inrta_tnsr_inv(3,3,1))

  ! Generate velocities from the  Maxwell-Boltzmann distribution [SQRT[(K mol) /(A^2 g)]]
  DO imol = 1,n_mol_tot
    wx(imol) = ran_norm(0.0d0,w_stdev(1))
    wy(imol) = ran_norm(0.0d0,w_stdev(2))
    wz(imol) = ran_norm(0.0d0,w_stdev(3))
  ENDDO
 
  ! Angular velocities [1/ps]
  wx = wx*dSQRT(Fc)
  wy = wy*dSQRT(Fc)
  wz = wz*dSQRT(Fc)

  RETURN
  
END SUBROUTINE set_velocity

