SUBROUTINE site_coord(imol,q0i,q1i,q2i,q3i)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: imol
  DOUBLE PRECISION :: q0i, q1i, q2i, q3i

  ! Local 
  INTEGER :: itype,isite
  DOUBLE PRECISION :: a11, a12, a13
  DOUBLE PRECISION :: a21, a22, a23
  DOUBLE PRECISION :: a31, a32, a33


  ! Compute rotation matrix elements (See Coutsias et al., J Comput Chem 25, 1849-1857,2004) (left handed rotation)
  a11 = q0i * q0i + q1i * q1i - q2i * q2i - q3i * q3i
  a12 = 2.0d0 * ( q1i * q2i - q0i * q3i )
  a13 = 2.0d0 * ( q1i * q3i + q0i * q2i )

  a21 = 2.0d0 * ( q1i * q2i + q0i * q3i )
  a22 = q0i * q0i  - q1i * q1i + q2i * q2i - q3i * q3i
  a23 = 2.0d0 * ( q2i * q3i - q0i * q1i )

  a31 = 2.0d0 * ( q1i * q3i - q0i * q2i )
  a32 = 2.0d0 * ( q2i * q3i + q0i * q1i )
  a33 = q0i * q0i - q1i * q1i - q2i * q2i + q3i * q3i

  ! Get imol's type
  itype = mol_type(imol)

  ! Calculate the Cartesian coordinates of the sites
  DO isite = 1,n_sites(itype)

    rx_s(isite,imol) = rx(imol) + a11 * rx_i(isite,itype) + a12 * ry_i(isite,itype) + a13 * rz_i(isite,itype)
    ry_s(isite,imol) = ry(imol) + a21 * rx_i(isite,itype) + a22 * ry_i(isite,itype) + a23 * rz_i(isite,itype)
    rz_s(isite,imol) = rz(imol) + a31 * rx_i(isite,itype) + a32 * ry_i(isite,itype) + a33 * rz_i(isite,itype)

  END DO


  RETURN

END SUBROUTINE site_coord

