SUBROUTINE stats(selector)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: selector

  ! Local
  INTEGER :: itype, jtype, i
  INTEGER :: iblk, istp, n_prod
  INTEGER :: n_total
  DOUBLE PRECISION :: engavg, volavg
  DOUBLE PRECISION :: engsd,  volsd
  DOUBLE PRECISION,DIMENSION(n_types) :: nmolavg, nmolsd, rhoavg, rhosd
  CHARACTER(LEN=15) :: filename
  

  ! Select statistics operation
  SELECT CASE (selector)

    ! Reset the statistics
    CASE(1)

      ! Zero the statistic arrays
      vol_stat = 0
      multimove_stat = 0

      ! Initialize block average
      blk_nmol = 0
      blk_rho = 0
      blk_eng = 0.0d0
      blk_vol = 0.0d0

    ! Dump to files and accumulate the statistics
    CASE(2)


       ! Loop over the molecule types
        DO itype = 1,n_types
          blk_nmol(istep,itype) = n_mol(itype)

          blk_rho(istep,itype) = n_mol(itype)/vol
        ENDDO

        ! Update the block volume
        blk_vol(istep) = vol

        ! Calculate the energy per molecule
        IF(n_mol_tot .GT. 0) THEN
          blk_eng(istep) = energy/n_mol_tot
        ELSE
          blk_eng(istep) = 0.0d0
        ENDIF


    ! Calculate the block averages and standard deviations
    CASE(3)

      ! Compute the sums

      ! Loop over the steps
      DO istp = 1,n_steps

          ! Loop over the molecule types
          DO itype = 1,n_types
            avg_nmol(iblock,itype) = avg_nmol(iblock,itype) +&
                                            &DBLE(blk_nmol(istp,itype))

            avg_rho(iblock,itype) = avg_rho(iblock,itype) +&
                                            &blk_rho(istp,itype)
          ENDDO

          avg_eng(iblock) = avg_eng(iblock) +&
                                   &blk_eng(istp)
          avg_vol(iblock) = avg_vol(iblock) +&
                                   &blk_vol(istp)

      ENDDO
  

      ! Compute the averages
    
        ! Loop over the molecule types
        DO itype = 1,n_types
          avg_nmol(iblock,itype) = avg_nmol(iblock,itype)/DBLE(n_steps)
          avg_rho(iblock,itype) = avg_rho(iblock,itype)/DBLE(n_steps)
        ENDDO
      
        avg_eng(iblock) = avg_eng(iblock)/DBLE(n_steps)
        avg_vol(iblock) = avg_vol(iblock)/DBLE(n_steps)


    ! Write the results to file
    CASE(4)

      
        ! Create an appropriate file name
        WRITE(filename,'(A,I0,A)')'avgbox',my_rank,'.out'

        ! Open the file 
        OPEN(UNIT=40,FILE=filename,STATUS='UNKNOWN',ACCESS='SEQUENTIAL',ACTION='WRITE')
    
          ! Write file header
          WRITE(40,'(A13,2A)',ADVANCE='NO')'','               Energy (J/mol)','           Volume (A3)   '
        
          ! Loop over the molecule types
          DO itype = 1,n_types
            WRITE(40,'(A11,A,I0,A7)', ADVANCE='NO')'','N mol',itype,'' 
            WRITE(40,'(A12,A,I0,A,A1)', ADVANCE='NO')'','rho',itype,' (mol/m3)',''
          ENDDO
         
          ! Write to advance to next line
          WRITE(40,*)

          DO iblk = 1,n_blocks
          
            ! Initialize the total number of molecules in the box
            n_total = 0

            ! Calculate the total number of molecules in the box
            DO itype = 1,n_types
              n_total = n_total + avg_nmol(iblk,itype)
            ENDDO


            ! Convert units and write data to file
            WRITE(40,'(A,I5,A,2F25.10)', ADVANCE='NO')&
              &'block ',iblk,' : ',R*avg_eng(iblk),avg_vol(iblk)

            ! Loop over the molecule types
            DO itype = 1,n_types
              WRITE(40,'(2F25.10)', ADVANCE='NO') avg_nmol(iblk,itype), avg_rho(iblk,itype)/Pr
            ENDDO
         
            ! Write to advance to next line
            WRITE(40,*)

          ENDDO

          ! Compute block averages for the equilibration blocks


          ! Initialize the averages
          engavg = 0.0d0
          engsd = 0.0d0
          volavg = 0.0d0
          volsd = 0.0d0
          n_total = 0.0d0
          nmolavg = 0.0d0
          nmolsd = 0.0d0
          rhoavg = 0.0d0
          rhosd = 0.0d0
          

          ! Loop over the equilibration blocks
          DO iblk = 1,n_equil
            
            ! Update sums
            engavg = engavg + avg_eng(iblk)
            volavg = volavg + avg_vol(iblk)

            DO itype = 1,n_types
              n_total = n_total + avg_nmol(iblk,itype) 
              nmolavg(itype) = nmolavg(itype) + avg_nmol(iblk,itype)
              rhoavg(itype) = rhoavg(itype) + avg_rho(iblk,itype)
            ENDDO
          
          ENDDO

          ! Compute the averages
          n_total = n_total/DBLE(n_equil)
          engavg = engavg/DBLE(n_equil)
          volavg = volavg/DBLE(n_equil)
          DO itype = 1,n_types
            nmolavg(itype) = nmolavg(itype)/DBLE(n_equil)
            rhoavg(itype)  = rhoavg(itype)/DBLE(n_equil)
          ENDDO

           
          ! Loop over the equilibration blocks
          DO iblk = 1,n_equil
            
            ! Update sums
            engsd = engsd + (avg_eng(iblk) - engavg)**2
            volsd = volsd + (avg_vol(iblk) - volavg)**2

            DO itype = 1,n_types
              nmolsd(itype) = nmolsd(itype) + (avg_nmol(iblk,itype) - nmolavg(itype))**2
              rhosd(itype) = rhosd(itype) + (avg_rho(iblk,itype) - rhoavg(itype))**2
            ENDDO
          ENDDO

          ! Compute the averages and standard deviations
          engsd = dSQRT(engsd/DBLE(n_equil))
          volsd = dSQRT(volsd/DBLE(n_equil))
   
          DO itype = 1,n_types
            nmolsd(itype) = dSQRT(nmolsd(itype)/DBLE(n_equil))
            rhosd(itype) = dSQRT(rhosd(itype)/DBLE(n_equil))
          ENDDO

          ! Convert units and write data to file
          WRITE(40,'(A,A,2F25.10)', ADVANCE='NO')'Equl Avgs','   : ',R*engavg, volavg

          ! Loop over the molecule types
          DO itype = 1,n_types
            WRITE(40,'(2F25.10)', ADVANCE='NO')nmolavg(itype),rhoavg(itype)/Pr
          ENDDO
         
          ! Write to advance to next line
          WRITE(40,*)

          ! Convert units and write data to file
          WRITE(40,'(A,A,2F25.10)', ADVANCE='NO')'Equl Stds','   : ',R*engsd, volsd

          ! Loop over the molecule types
          DO itype = 1,n_types
            WRITE(40,'(2F25.10)', ADVANCE='NO')nmolsd(itype),rhosd(itype)/Pr
          ENDDO
         
          ! Write to advance to next line
          WRITE(40,*)




          ! Compute block averages for the production blocks

          ! Compute the number of production blocks
          n_prod = n_blocks-n_equil

          ! Initialize the averages
          engavg = 0.0d0
          engsd = 0.0d0
          volavg = 0.0d0
          volsd = 0.0d0
          n_total = 0.0d0
          nmolavg = 0.0d0
          nmolsd = 0.0d0
          rhoavg = 0.0d0
          rhosd = 0.0d0

          ! Loop over the equilibration blocks
          DO iblk = n_equil+1,n_blocks
            
            ! Update sums
            engavg = engavg + avg_eng(iblk)
            volavg = volavg + avg_vol(iblk)

            DO itype = 1,n_types
              n_total = n_total + avg_nmol(iblk,itype) 
              nmolavg(itype) = nmolavg(itype) + avg_nmol(iblk,itype)
              rhoavg(itype) = rhoavg(itype) + avg_rho(iblk,itype)
            ENDDO
          ENDDO

          ! Compute the averages
          n_total = n_total/DBLE(n_prod)
          engavg = engavg/DBLE(n_prod)
          volavg = volavg/DBLE(n_prod)
          DO itype = 1,n_types
            nmolavg(itype) = nmolavg(itype)/DBLE(n_prod)
            rhoavg(itype) = rhoavg(itype)/DBLE(n_prod)
          ENDDO

           
          ! Loop over the equilibration blocks
          DO iblk = n_equil+1,n_blocks
            
            ! Update sums
            engsd = engsd + (avg_eng(iblk) - engavg)**2
            volsd = volsd + (avg_vol(iblk) - volavg)**2

            DO itype = 1,n_types 
              nmolsd(itype) = nmolsd(itype) + (avg_nmol(iblk,itype) - nmolavg(itype))**2
              rhosd(itype) = rhosd(itype) + (avg_rho(iblk,itype) - rhoavg(itype))**2
            ENDDO
          ENDDO

          ! Compute the averages and standard deviations
          engsd = dSQRT(engsd/DBLE(n_prod))
          volsd = dSQRT(volsd/DBLE(n_prod))
    
          DO itype = 1,n_types
            nmolsd(itype) = dSQRT(nmolsd(itype)/DBLE(n_prod))
            rhosd(itype) = dSQRT(rhosd(itype)/DBLE(n_prod))
          ENDDO


          ! Convert units and write data to file
          WRITE(40,'(A,A,2F25.10)', ADVANCE='NO')'Prod Avgs','   : ',R*engavg,volavg

          ! Loop over the molecule types
          DO itype = 1,n_types
            WRITE(40,'(2F25.10)', ADVANCE='NO')nmolavg(itype),rhoavg(itype)/Pr
          ENDDO

          ! Write to advance to next line
          WRITE(40,*)

          ! Convert units and write data to file
          WRITE(40,'(A,A,2F25.10)', ADVANCE='NO')'Prod Stds','   : ',R*engsd,volsd

          ! Loop over the molecule types
          DO itype = 1,n_types
            WRITE(40,'(2F25.10)', ADVANCE='NO')nmolsd(itype),rhosd(itype)/Pr
          ENDDO

          ! Write to advance to next line
          WRITE(40,*)


        ! Close the output file
        CLOSE(UNIT=40)


    ! Throw an error
    CASE DEFAULT

      WRITE(*,*) 'FATAL ERROR: INVALID FLAG TYPE IN STATS SUBROUTINE'
      STOP

  END SELECT


  RETURN

END SUBROUTINE stats
