SUBROUTINE volchng_npt

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Local
  INTEGER :: imol,i,itype,jtype,ierr 
  DOUBLE PRECISION :: eng_new, eng_old
  DOUBLE PRECISION,DIMENSION(3,n_mol_max) :: frc_new
  DOUBLE PRECISION,DIMENSION(3,n_mol_max) :: trq_new
  DOUBLE PRECISION :: vol_old
  DOUBLE PRECISION :: box_old
  DOUBLE PRECISION :: lnvol, scalef, boltzf, arg
  DOUBLE PRECISION,PARAMETER :: Pconv = 1.0d0/Pb
  DOUBLE PRECISION,DIMENSION(3) :: cv_data
  DOUBLE PRECISION :: cv_eng_chng, lrc_eng_chng
  LOGICAL :: ACCPT


  ! Assume the move is rejected
  ACCPT = .FALSE.

  ! Initialize the CV energy change to zero
  cv_eng_chng = 0.0d0

  ! Store old energy, volume and box dimension
  eng_old = energy
  vol_old = vol
  box_old = box(1)
 
  ! Compute random volume displacement in ln(V)
  lnvol = dLOG(vol_old) + (random(idum) - 0.5d0)*max_vol

  ! Compute the new volumes, conserving the total volume
  vol = dEXP(lnvol)
  
  ! Compute the new box dimensions
  DO i = 1,3
    box(i) = vol**(1.0d0/3.0d0)
  ENDDO

  ! Calculate the scale factor for the box
  scalef = box(1)/box_old

  ! Check if the potential cutoff is too small
  IF(MAXVAL(rmax) .GT. 0.5d0*box(1)) THEN
    WRITE(*,*) 'FATAL ERROR: SIMULATION CELL HAS BECOME TO SMALL FOR THE POTENTIAL CUTOFF'
    STOP
  ENDIF
  
  ! Loop over the molecules in the box
  DO imol = 1,n_mol_tot
    
    ! Scale the molecules center of mass position vector
    rx(imol) = rx(imol)*scalef
    ry(imol) = ry(imol)*scalef
    rz(imol) = rz(imol)*scalef

    ! Calculate the site position
    CALL site_coord(imol,q0(imol),q1(imol),q2(imol),q3(imol))

  ENDDO

  ! If Ewald, setup the k-vectors using the new box dimensions
  IF(ewld .EQ. 1) CALL set_ewld(1)


  ! Update the number of volume change trials
  vol_stat(1) = vol_stat(1) + 1

  ! If running weighted ensemble simulation, calculate the weights
  IF(wensmbl .EQ. 1) THEN

    CALL weighted_ensmbl(2,3,imol,cv_data,cv_eng_chng)

  ENDIF

  ! Calculate the total energy for each box (Note: Includes Ewald already)
  CALL engforce_total(eng_new,frc_new,trq_new)

  ! Evaluate energy change due to long-range correction
  lrc_eng_chng = (1.0d0/vol - 1.0d0/vol_old)*lrc_fact_npt

  ! Calculate the argument factor
  arg = (eng_new-eng_old)&
           &+lrc_eng_chng&
           &+Pconv*press*(vol-vol_old)&
           &-(temp(my_rank)*(DBLE(n_mol_tot) + 1.0d0)*dLOG(vol/vol_old))


  ! Check for large energies to avoid underflow
  IF((arg + cv_eng_chng)/temp(my_rank) .GT. 180.0d0) GOTO 10

  ! Apply the Metropolis criteria
  IF((arg + cv_eng_chng) .LT. 0.0d0) THEN
    ACCPT = .TRUE.
  ELSE
    boltzf = MIN(1.0d0,dEXP(-(arg+cv_eng_chng)/temp(my_rank)))
    IF(random(idum) .LT. boltzf) ACCPT = .TRUE.
  ENDIF


  ! Continue here if overlap or large energy
10 CONTINUE


  ! If the move is accepted
  IF(ACCPT) THEN
  
    ! Update the number of accepted volume displacements
    vol_stat(2) = vol_stat(2) + 1

    ! Update the energy and net force and torque
    energy = eng_new
    force = frc_new
    torque = trq_new

    ! Update the CVs and CV energies
    IF(wensmbl .EQ. 1) CALL weighted_ensmbl(3,3,imol,cv_data,0.0d0)

  ELSE

    ! Calculate the scale factor 
    scalef = box_old/vol**(1.0d0/3.0d0)

    ! Restore the old box volume
    vol = vol_old

    ! Restore the old box dimensions
    DO i = 1,3
      box(i) = box_old 
    ENDDO

    ! Loop over the molecules in the box
    DO imol = 1,n_mol_tot
    
      ! Rescale the molecules center of mass position vector
      rx(imol) = rx(imol)*scalef
      ry(imol) = ry(imol)*scalef
      rz(imol) = rz(imol)*scalef

      ! Calculate the site position
      CALL site_coord(imol,q0(imol),q1(imol),q2(imol),q3(imol))

    ENDDO

     ! If Ewald, reset the k-vectors
     IF(ewld .EQ. 1) CALL set_ewld(1)
 
  ENDIF


  RETURN

END SUBROUTINE volchng_npt
