SUBROUTINE write_config

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Local
  INTEGER :: imol,itype,isite,n_sites_tot
  CHARACTER(LEN=25) :: filename
  CHARACTER(LEN=10) :: mol


  ! Write com positions and quaternions

  ! Create and appropriate file name
  WRITE(filename,'(A,I0,A,I0,A)') 'config_b',my_rank,'_',iblock,'.xyz'

  OPEN(UNIT=30,FILE=filename,STATUS='UNKNOWN',ACCESS='SEQUENTIAL',ACTION='WRITE')
  
    ! Write the total number of molecules
    WRITE(30,'(I10)', ADVANCE='NO') n_mol_tot
    
    ! Write the number of molecules of each component type
    DO itype = 1,n_types
      WRITE(30,'(I10)',ADVANCE='NO') n_mol(itype)
    ENDDO  

    ! Write to advance to the next line
    WRITE(30,*)

    ! Write the box dimensions
    WRITE(30,'(3F17.9)') box(1),box(2),box(3) 

    ! Write the coordinates to file (sorting by type)
    DO itype = 1,n_types

      ! Loop over the molecules
      DO imol = 1,n_mol_tot

        ! Check that it is the right type
        IF(mol_type(imol) .EQ. itype) THEN
        
          ! Write the type and coordinates to file
          WRITE(30,'(A4,7F17.9)') mol_name(itype),rx(imol),ry(imol),rz(imol),&
                                  &q0(imol),q1(imol),q2(imol),q3(imol)     

        ENDIF
      ENDDO
    ENDDO

  CLOSE(UNIT=30)


  ! Write site positions
  
  ! Initialize the total number of sites
  n_sites_tot = 0

  ! Calculate the total number of atomic sites  
  DO itype = 1,n_types
    n_sites_tot = n_sites_tot + n_sites(itype)*n_mol(itype)
  ENDDO

  ! Create and appropriate file name
  WRITE(filename,'(A,I0,A,I0,A)') 'config_b',my_rank,'_',iblock,'_exp.xyz'

  OPEN(UNIT=35,FILE=filename,STATUS='UNKNOWN',ACCESS='SEQUENTIAL',ACTION='WRITE')

     ! Write the total number of molecule sites
    WRITE(35,'(I10)', ADVANCE='NO') n_sites_tot

    ! Write the total number of molecules
    WRITE(35,'(I10)', ADVANCE='NO') n_mol_tot

    ! Write the number of molecules of each component type
    DO itype = 1,n_types
      WRITE(35,'(I10)',ADVANCE='NO') n_mol(itype)
    ENDDO

    ! Write to advance to the next line
    WRITE(35,*)

    ! Write the box dimensions
    WRITE(35,'(3F17.9)') box(1),box(2),box(3)

    ! Write the coordinates to file (sorting by type)
    DO itype = 1,n_types

      ! Loop over the molecules
      DO imol = 1,n_mol_tot

        ! Check that it is the right type
        IF(mol_type(imol) .EQ. itype) THEN

          ! Loop over imol's sites 
          DO isite = 1,n_sites(itype)

            ! Write the type and coordinates to file
            WRITE(35,'(A4,3F17.9)') site_name(isite,itype),rx_s(isite,imol),ry_s(isite,imol),rz_s(isite,imol)

          ENDDO
        ENDIF
      ENDDO
    ENDDO

  CLOSE(UNIT=35)


  RETURN

END SUBROUTINE write_config
